#    Copyright 2022 Volikov Alexander <ab.volikov@gmail.com>
#
#    This file is part of nomspectra. 
#
#    nomspectra is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nomspectra is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nomspectra.  If not, see <http://www.gnu.org/licenses/>.

from typing import Dict, Optional
import copy
import numpy as np
import pandas as pd
from tqdm import tqdm
from scipy.signal import find_peaks

from .spectrum import Spectrum
from .brutto import brutto_gen
from .metadata import MetaData


def assign_by_tmds(
    spec: "Spectrum", 
    tmds_spec: Optional["Tmds"] = None,
    tmds_brutto_dict: Optional[Dict] = None, 
    rel_error: float = 3,
    p = 0.2,
    max_num: Optional[int] = None,
    C13_filter: bool = True
    ) -> "Spectrum":
    '''
    Assigne brutto formulas by TMDS

    Additianal assignment of masses that can't be done with usual methods

    Parameters
    ----------
    spec: Spectrum object
        Mass spectrum for assign by tmds
    tmds_spec: Tmds object
        Optional. if None generate tmds spectr with default parameters
        Tmds object, include table with most intensity mass difference
    brutto_dict: dict
        Optional. Deafault None.
        Custom Dictonary for generate brutto table.
        Example: {'C':(-1,20),'H':(-4,40), 'O':(-1,20),'N':(-1,2)}
    abs_error: float
        Optional, default 1 ppm. Relative error for assign peaks by massdif
    p: float
        Optional. Default 0.2. 
        Relative intensity coefficient for treshold tmds spectrum
    max_num: int
        Optional. Max mass diff numbers
    C13_filter: bool
        Use only peaks with C13 isotope peak for generate tmds. Default True.

    Return
    ------
    Spectrum
        Assigned by tmds masses

    Reference
    ---------
    Kunenkov, Erast V., et al. "Total mass difference 
    statistics algorithm: a new approach to identification 
    of high-mass building blocks in electrospray ionization 
    Fourier transform ion cyclotron mass spectrometry data 
    of natural organic matter." 
    Analytical chemistry 81.24 (2009): 10106-10115.
    '''
    
    if "assign" not in spec.table:
        raise Exception("Spectrum is not assigned")

    spec = spec.copy()

    #calculstae tmds table
    if tmds_spec is None:
        tmds_spec = Tmds(spec=spec).calc(p=p, C13_filter=C13_filter) #by varifiy p-value we can choose how much mass-diff we will take
        tmds_spec = tmds_spec.assign(max_num=max_num, brutto_dict=tmds_brutto_dict)
        tmds_spec = tmds_spec.calc_mass()

    #prepare tmds table
    tmds = tmds_spec.table.sort_values(by='intensity', ascending=False).reset_index(drop=True)
    tmds = tmds.loc[tmds['intensity'] > p].sort_values(by='mass', ascending=True).reset_index(drop=True)
    elem_tmds = tmds_spec.find_elements()

    #prepare spec table
    assign_false = copy.deepcopy(spec.table.loc[spec.table['assign'] == False]).reset_index(drop=True)
    assign_true = copy.deepcopy(spec.table.loc[spec.table['assign'] == True]).reset_index(drop=True)
    masses = assign_true['mass'].values
    elem_spec = spec.find_elements()
    
    
    #Check that all elements in tmds also in spec
    if len(set(elem_tmds)-set(elem_spec)) > 0:
        raise Exception(f"All elements in tmds spectrum must be in regular spectrum too. But {(set(elem_tmds)-set(elem_spec))} not in spectrum")
    for i in set(elem_spec)-set(elem_tmds):
        tmds[i] = 0

    mass_dif_num = len(tmds)
    min_mass = np.min(masses)

    for i, row_tmds in tqdm(tmds.iterrows(), total=mass_dif_num):

        mass_shift = - row_tmds['calc_mass']
        
        for index, row in assign_false.iterrows():
            if row['assign'] == True:
                continue
                    
            mass = row["mass"] + mass_shift
            if mass < min_mass:
                continue

            idx = np.searchsorted(masses, mass, side='left')
            if idx > 0 and (idx == len(masses) or np.fabs(mass - masses[idx - 1]) < np.fabs(mass - masses[idx])):
                idx -= 1
                
            if np.fabs(masses[idx] - mass) / mass * 1e6 <= rel_error:
                assign_false.loc[index,'assign'] = True

                for el in elem_spec:
                    assign_false.loc[index,el] = row_tmds[el] + assign_true.loc[idx,el]

    assign_true = pd.concat([assign_true, assign_false], ignore_index=True).sort_values(by='mass').reset_index(drop=True)
    
    out = Spectrum(assign_true)
    out = out.calc_mass()

    out.table=out.table[out.table['calc_mass'].isnull() | ~out.table[out.table['calc_mass'].notnull()].duplicated(subset='calc_mass',keep='first')] 
    spec.table = out.table.sort_values(by='mass').reset_index(drop=True)

    spec.metadata.add({'assigned_by_tmds':True})
    
    return spec


class Tmds(Spectrum):
    """
    A class for calculate TMDS (Total mass difference 
    statistics) spectrum

    Reference
    ---------
    Kunenkov, Erast V., et al. "Total mass difference 
    statistics algorithm: a new approach to identification 
    of high-mass building blocks in electrospray ionization 
    Fourier transform ion cyclotron mass spectrometry data 
    of natural organic matter." 
    Analytical chemistry 81.24 (2009): 10106-10115.
    """

    def __init__(self, 
                spec: Optional["Spectrum"] = None, 
                table: Optional[pd.DataFrame] = None) -> None:
        """
        Parameters
        ----------
        table: pandas Datarame
            tmds spectrum - mass, intensity and caclulatedd parameters
        metadata: MetaData
            Metadata object that consist dictonary of metadata
        """

        if spec is None:
            self.metadata = MetaData()
            self.spec = Spectrum()
        else:
            self.spec = spec.copy()
            self.metadata = spec.metadata
        self.metadata['type'] = 'Tmds'

        if table is None:
            self.table = pd.DataFrame()
        else:
            self.table = table

        super().__init__(self.table, self.metadata)

    def calc(
        self,
        other: Optional["Spectrum"] = None,
        p: float = 0.2,
        wide: int = 10,
        C13_filter:bool = True,
        ) -> "Tmds":
        """
        Total mass difference statistic calculation 

        Parameters
        ----------
        other: Spectrum object
            Optional. If None, TMDS will call by self.
        p: float
            Minimum relative intensity for taking mass-difference. Default 0.2.
        wide: int
            Minimum interval in 0.001*wide Da of peaks finding. Default 10.
        C13_filter: bool
            Use only peaks that have C13 isotope peak. Default True

        Return
        ------
        Tmds
        """

        spec = copy.deepcopy(self.spec)
        if other is None:
            spec2 = copy.deepcopy(self.spec)
        else:
            spec2 = copy.deepcopy(other)

        if C13_filter:
            spec = spec.filter_by_C13(remove=True)
            spec2 = spec2.filter_by_C13(remove=True)
        else:
            spec = spec.drop_unassigned()
            spec2 = spec2.drop_unassigned()

        masses = spec.table['mass'].values
        masses2 = spec2.table['mass'].values

        mass_num = len(masses)
        mass_num2 = len(masses2)

        if mass_num <2 or mass_num2 < 2:
            raise Exception(f"Too low number of assigned peaks")

        mdiff = np.zeros((mass_num, mass_num2), dtype=float)
        for x in range(mass_num):
            for y in range(x, mass_num2):
                dif = np.fabs(masses[x]-masses2[y])
                if dif < 300:
                    mdiff[x,y] = dif

        mdiff = np.round(mdiff, 3)
        unique, counts = np.unique(mdiff, return_counts=True)
        counts[0] = 0

        tmds_spec = pd.DataFrame()
        tmds_spec['mass'] = unique
        tmds_spec['count'] = counts
        tmds_spec['intensity'] = tmds_spec['count']/mass_num
        tmds_spec = tmds_spec.sort_values(by='mass').reset_index(drop=True)

        value_zero = set([i/1000 for i in range (0, 300000)]) - set (unique)
        unique = np.append(unique, np.array(list(value_zero)))
        counts = np.append(counts, np.zeros(len(value_zero), dtype=float))

        peaks, properties = find_peaks(tmds_spec['intensity'], distance=wide, prominence=p/2)
        prob = []
        for peak in peaks:
            prob.append(tmds_spec.loc[peak-5:peak+5,'intensity'].sum())
        tmds_spec = tmds_spec.loc[peaks].reset_index(drop=True)
        tmds_spec['intensity'] = prob
        tmds_spec = tmds_spec.loc[tmds_spec['intensity'] > p]

        if len(tmds_spec) < 0:
            raise Exception(f"There isn't mass diff mass, decrease p-value")

        self.table = tmds_spec

        return self
    
    def calc_by_brutto(self) -> "Tmds":

        """
        Calculate self difference by calculated mass from brutto

        Return
        ------
        Tmds
        """

        mass = self.spec.drop_unassigned().calc_error().table['calc_mass'].values
        massl = len(mass)
        mdiff = np.zeros((massl, massl), dtype=float)
        for x in range(massl):
            for y in range(x, massl):
                mdiff[x,y] = np.fabs(mass[x]-mass[y])

        mdiff = np.round(mdiff, 6)
        unique, counts = np.unique(mdiff, return_counts=True)
        counts[0] = 0

        diff_spec = pd.DataFrame()
        diff_spec['mass'] = unique
        diff_spec['count'] = counts
        diff_spec['intensity'] = diff_spec['count']/massl
        diff_spec = diff_spec.sort_values(by='mass').reset_index(drop=True)

        self.table = diff_spec

        return self

    def assign(
        self,
        generated_bruttos_table: Optional[pd.DataFrame] = None,
        error: float = 0.001,
        brutto_dict: Optional[dict] = None,
        max_num: Optional[int] = None
        ) -> "Tmds":

        """
        Finding the nearest mass in generated_bruttos_table

        Parameters
        ----------
        generated_bruttos_table: pandas DataFrame 
            Optional. with column 'mass' and elements, should be sorted by 'mass'
        error: float
            Optional. Default 0.001. 
            absolute error iin Da for assign formulas
        brutto_dict: dict
            Optional, default {'C':(-1,20),'H':(-4,40), 'O':(-1,20),'N':(-1,2)}
            generate brutto table if generated_bruttos_table is None.
        max_num: int
            Optional. Default 100
        
        Return
        ------
        Tmds
        """

        if brutto_dict is None:
            brutto_dict = {'C':(-1,20),'H':(-4,40), 'O':(-1,20),'N':(-1,2)}

        if generated_bruttos_table is None:
            generated_bruttos_table = brutto_gen(brutto_dict, rules=False)
            generated_bruttos_table = generated_bruttos_table.loc[generated_bruttos_table['mass'] > 0]

        res = super().assign(generated_bruttos_table=generated_bruttos_table, abs_error=error, sign='0').drop_unassigned().table

        if max_num is not None and len(res) > max_num:
            res = res.sort_values(by='intensity', ascending=False).reset_index(drop=True)
            res = res.loc[:max_num].reset_index(drop=True)
            res = res.sort_values(by='mass').reset_index(drop=True)
        
        self.table = res

        return self


if __name__ == '__main__':
    pass


#    Copyright 2022 Volikov Alexander <ab.volikov@gmail.com>
#
#    This file is part of nomspectra. 
#
#    nomspectra is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nomspectra is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nomspectra.  If not, see <http://www.gnu.org/licenses/>.

from typing import Optional, Tuple
import copy
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from scipy.signal import savgol_filter
from scipy.interpolate import interp1d
import scipy.stats as st

from .brutto import gen_from_brutto
from .spectrum import Spectrum

def recallibrate(spec: "Spectrum", 
                error_table: Optional["ErrorTable"] = None, 
                how: str = 'assign',
                mode: str = '-',
                draw: bool = True) -> "Spectrum":
    '''
    Recallibrate spectrum

    Fine reccalabration based on self (assign or mass differnce map)
    or with external ethalon spectrum

    Parameters
    ----------
    spec: Spectrum object
        Mass spectrum for recallibration
    error_table: ErrorTable object
        Optional. If None - calculate for spec. 
        ErrorTable object contain table error in ppm for mass, default 100 string            
    how: {'assign', 'mdm', filename} 
        Optional. Default 'assign'.
        If error_table is None we can choose how to recalculate.
        'assign' - by assign error, default.
        'mdm' - by calculation mass-difference map.
        filename - path to etalon spectrum, treated and saved by nomspectra
    mode: str
        Optional. Default '-' negative mode. May be +, - or 0
    draw: bool
        Plot error (fit of KDM)

    Returns
    -------
    Spectrum
    '''

    spec = spec.copy()

    if error_table is None:
        if how == 'assign':
            error_table = ErrorTable().assign_error(spec, show_map=draw, mode=mode)
        elif how == 'mdm':
            error_table = ErrorTable().massdiff_error(spec, show_map=draw)
        else:
            etalon = Spectrum.read_csv(filename=how)
            error_table = ErrorTable().etalon_error(spec=spec, etalon=etalon, show_map=draw)

    err = copy.deepcopy(error_table.table)
    spec.table = spec.table.reset_index(drop=True)
    wide = len(err)

    min_mass = err['mass'].min()
    max_mass = err['mass'].max()
    a = np.linspace(min_mass, max_mass, wide+1)

    for i in range(wide):
        for ind in spec.table.loc[(spec.table['mass']>a[i]) & (spec.table['mass']<=a[i+1])].index:
            mass = spec.table.loc[ind, 'mass']
            e = mass * err.loc[i, 'ppm'] / 1000000
            spec.table.loc[ind, 'mass'] = spec.table.loc[ind, 'mass'] + e

    spec.metadata.add({'recallibrate':how})

    return spec


class ErrorTable(object):
    """
    A class used to recallibrate mass spectrum
    """

    def __init__(
            self,
            table: Optional[pd.DataFrame] = None,
    ) -> None:
        """
        Init ErrorTable object

        Parameters
        ----------
        table : pandas Datarame
            consist error table: error in ppm for mass
        """
        
        self.table = table

    @staticmethod
    def md_error_map(
        spec: "Spectrum", 
        ppm: float = 3
        ) -> pd.DataFrame:
        '''
        Calculate mass differnce map

        Parameters
        ----------
        spec: Spectrum
            data
        ppm: float
            Optional. Default 3.
            Permissible error in ppm
        show_map: bool
            Optional. Default False.
            Show error in ppm versus mass

        Return
        ------
        Pandas Dataframe
        '''

        #common neutral mass loses: CH2, CO, CH2O, C2HO, H2O, CO2
        df = pd.DataFrame({ 'C':[1,1,1,2,0,1],
                            'H':[2,0,2,1,2,0],
                            'O':[0,1,1,1,1,2]})

        dif_masses = gen_from_brutto(df)['calc_mass'].values
        dif = np.unique([dif_masses*i for i in range(1,10)])

        data = copy.deepcopy(spec.table)
        masses = data['mass'].values

        data = data.sort_values(by='intensity', ascending=False).reset_index(drop=True)
        if len(data) > 1000:
            data = data[:1000]
        data = data.sort_values(by='mass').reset_index(drop=True)

        data_error = [] #array for new data

        for index, row in data.iterrows(): #take every mass in list
            
            mass = row["mass"]

            for i in dif:
                mz = mass + i #massdif

                idx = np.searchsorted(masses, mz, side='left')                
                if idx > 0 and (idx == len(masses) or np.fabs(mz - masses[idx - 1]) < np.fabs(mz - masses[idx])):
                    idx -= 1

                if np.fabs(masses[idx] - mz) / mz * 1e6 <= ppm:
                    data_error.append([mass, (masses[idx] - mz)/mz*1000000])
        
        df_error = pd.DataFrame(data = data_error, columns=['mass', 'ppm' ])

        return df_error
    
    @staticmethod
    def fit_kernel(
        f: np.array,
        mass: np.array,
        err_ppm: float = 3,
        show_map: bool = True) -> pd.DataFrame:
        '''
        Fit max intesity of kernel density map

        Parameters
        ----------
        f: np.array
            keerndel density map in numpy array 100*100
        show_map: bool
            Optional. Default true.
            Plot how fit kernel

        Return
        ------
        Pandas Dataframe
        '''

        df = pd.DataFrame(f, index=np.linspace(err_ppm,-err_ppm,100))

        out = []
        for i in df.columns:
            max_kernel = df[i].quantile(q=0.95)
            ppm = df.loc[df[i] > max_kernel].index.values
            out.append([i, np.mean(ppm)])
        kde_err = pd.DataFrame(data=out, columns=['i','ppm'])
        
        #smooth data
        kde_err['ppm'] = savgol_filter(kde_err['ppm'], 31,3)
        
        xmin = min(mass)
        xmax = max(mass)
        
        kde_err['mass'] = np.linspace(xmin, xmax, 100)

        ymin = -err_ppm
        ymax = err_ppm

        if show_map:
            fig = plt.figure(figsize=(4,4), dpi=75)
            ax = fig.gca()
            ax.set_xlim(xmin, xmax)
            ax.set_ylim(ymin, ymax)
            ax.imshow(df, extent=[xmin, xmax, ymin, ymax], aspect='auto')
            ax.plot(kde_err['mass'], kde_err['ppm'], c='r')
            ax.set_xlabel('m/z, Da')
            ax.set_ylabel('error, ppm')      

        return kde_err

    @staticmethod
    def kernel_density_map(
        df_error: pd.DataFrame, 
        ppm: float = 3, 
        ) -> np.array:
        '''
        Calculate and plot kernel density map 100*100 for data

        Parameters
        ----------
        df_error: pd.Dataframe
            error_table for generate kerle density map
        ppm: float
            Optional. Default 3.
            treshould for generate
        show_map: bool
            Optional. Default False. plot kde

        Return
        ------
        numpy array
        '''
        
        x = np.array(df_error['mass'])
        y = np.array(df_error['ppm'])

        xmin = min(x) 
        xmax = max(x) 

        ymin = -ppm 
        ymax = ppm 

        xx, yy = np.mgrid[xmin:xmax:100j, ymin:ymax:100j]

        positions = np.vstack([xx.ravel(), yy.ravel()])
        values = np.vstack([x, y])
        kernel = st.gaussian_kde(values)
        kdm = np.reshape(kernel(positions).T, xx.shape)
        kdm = np.rot90(kdm)
        
        return kdm

    @staticmethod
    def assign_error(
        spec: Spectrum,
        ppm: float = 3,
        brutto_dict = {'C':(4,30), 'H':(4,60), 'O':(0,20)},
        mode = '-',
        show_map: bool = True):
        '''
        Recallibrate by assign error

        Parameters
        ----------
        spec: Spectrum object
            Initial mass spectrum for recallibrate
        ppm: float
            Permissible relative error in callibrate error. Default 3.
        brutto_dict: dict
            Dictonary with elements ranges for assignment
        mode: str
            Optional. Default '-' negative mode. May be +, - or 0
        show_error: bool
            Optional. Default True. Show process 

        Return
        ------
        ErrorTable
        '''

        spectr = copy.deepcopy(spec)
        spectr = spectr.assign(rel_error=ppm, brutto_dict=brutto_dict, sign=mode)
        spectr = spectr.calc_mass().calc_error()

        error_table = spectr.table
        error_table = error_table.loc[:,['mass','rel_error']]
        error_table.columns = ['mass', 'ppm']
        error_table['ppm'] = - error_table['ppm']
        error_table = error_table.dropna()

        kdm = ErrorTable.kernel_density_map(df_error = error_table)
        err = ErrorTable.fit_kernel(f=kdm, 
                            show_map=show_map, 
                            mass=spectr.drop_unassigned().table['mass'].values)
        err = ErrorTable(err).extrapolate((spec.table['mass'].min(), spec.table['mass'].max()))

        return err

    @staticmethod
    def massdiff_error( spec: Spectrum,
                        show_map:bool = True):
        '''
        Self-recallibration of mass-spectra by mass-difference map

        Parameters
        -----------
        spec: Spectrum object
            Initial mass spectrum for recallibrate
        show_error: bool
            Optional. Default True. Show process 

        Return
        -------
        ErrorTable

        References
        ----------
        Smirnov, K. S., Forcisi, S., Moritz, F., Lucio, M., & Schmitt-Kopplin, P. 
        (2019). Mass difference maps and their application for the 
        recalibration of mass spectrometric data in nontargeted metabolomics. 
        Analytical chemistry, 91(5), 3350-3358. 
        '''

        spec = copy.deepcopy(spec)
        mde = ErrorTable.md_error_map(spec = spec)
        kdm = ErrorTable.kernel_density_map(df_error=mde)
        err = ErrorTable.fit_kernel(f=kdm, show_map=show_map, mass=spec.table['mass'].values)
        
        err = ErrorTable(err)
        err.table['ppm'] = err.table['ppm'] - err.table.loc[0, 'ppm']

        return err

    @staticmethod
    def etalon_error(spec: "Spectrum", #initial masspectr
                    etalon: "Spectrum", #etalon massspectr
                    quart: float = 0.9, #treshold by quartile
                    ppm: float = 3,#treshold by ppm
                    show_map: bool = True
                    ): 
        '''
        Recallibrate by etalon

        Parameters
        ----------
        spec: Spectrum object
            Initial mass spectrum for recallibrate
        etalon: Spectrum object
            Etalon mass spectrum
        quart: float
            Optionaly. by default it is 0.9. 
            Usualy it is enough for good callibration
            Quartile, which will be taken for calc recallibrate error
        ppm: float
            Optionaly. Default 3.
            permissible relative error in ppm for seak peak in etalon
        show_map: bool
            Optional. Default True. Show process 

        Return
        ------
        ErrorTable
        '''

        et = copy.deepcopy(etalon.table)['mass'].to_list()
        df = copy.deepcopy(spec.table)

        min_mass = df['mass'].min()
        max_mass = df['mass'].max()
        a = np.linspace(min_mass,max_mass,101)

        treshold = df['intensity'].quantile(quart)
        df = df.loc[df['intensity'] > treshold].reset_index(drop = True)
        df['cal'] = 0 #column for check

        #fill data massiv with correct mass
        for i in range(0,len(df)):
            min_mass = df.loc[i, 'mass']*(1 - ppm/1000000)
            max_mass = df.loc[i, 'mass']*(1 + ppm/1000000)
            for mass in et:
                try:
                    if mass > min_mass and mass < max_mass:
                        df.loc[i, 'cal'] = mass
                except:
                    pass
        
        # take just assigned peaks
        df = df.loc[df['cal']>0]
        #calc error and mean error
        df['dif'] = df['cal'] - df['mass']
        df['ppm']=df['dif']/df['mass']*1000000

        error_table = df.loc[:,['mass','ppm']]
        error_table = error_table.dropna()

        kdm = ErrorTable.kernel_density_map(df_error = error_table)
        err = ErrorTable.fit_kernel(f=kdm, show_map=show_map, mass=spec.table['mass'].values)

        return ErrorTable(err)

    def extrapolate(self, ranges:Tuple[float, float] = None) -> "ErrorTable":
        """
        Extrapolate error data

        Parameters
        ----------
        ranges: Tuple(numeric, numeric)
            Optionaly. Default None - all width of mass in error table.
            For which diaposone of mass extrapolate data

        Return
        ------
        ErrorTable
        """
        
        if ranges is None:
            ranges = [self.table['mass'].min(), self.table['mass'].max()]

        interpolation_range = np.linspace(ranges[0], ranges[1], 100)
        linear_interp = interp1d(self.table['mass'], self.table['ppm'],  bounds_error=False, fill_value='extrapolate')
        linear_results = linear_interp(interpolation_range)
        err = pd.DataFrame()
        err ['mass'] = interpolation_range
        err ['ppm'] = linear_results

        return ErrorTable(err)

    def show_error(self) -> None:
        """
        Plot error map from ErrorTable class data
        """

        fig, ax = plt.subplots(figsize=(4,4), dpi=75)
        ax.plot(self.table['mass'], self.table['ppm'])
        ax.set_xlabel('m/z, Da')
        ax.set_ylabel('error, ppm')

if __name__ == '__main__':
    pass


#    Copyright 2019-2021 Rukhovich Gleb
#    Copyright 2022 Volikov Alexander <ab.volikov@gmail.com>
#
#    This file is part of nomspectra. 
#
#    nomspectra is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nomspectra is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nomspectra.  If not, see <http://www.gnu.org/licenses/>.

import os
from pathlib import Path
from typing import List, Sequence, Union, Optional
import copy
import json
from collections import UserList
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns

from .spectrum import Spectrum


class SpectrumList(UserList):
    """
    Class for work list of Spectrums objects
    inheritan from list class with some extra features.
    Store list of Spectrum objects
    """

    def __init__(self, spectra: Optional[List["Spectrum"]] = []):
        """
        init SpectrumList Class
        
        Parameters
        ----------
        spectra: Sequence[Spectrum]
            list of Spectrum objects
        """

        t = type(Spectrum())
        for spec in spectra:
            if isinstance(spec, t) == False:
                raise Exception(f'SpectrumList must contain only Spectrum objects, not {type(spec)}')

        super().__init__(spectra)
        self.data: List[Spectrum]

    @staticmethod
    def read_json(filename: Union[Path, str]) -> "SpectrumList":
        """
        Read SpectrumList from json

        Parameters
        ----------
        filename: str
            path to SpectrumList json file, absoulute or relative

        Return
        ------
        SpectrumList
        """

        specs = SpectrumList()

        with open(filename, 'rb') as data:
            res = json.load(data)

        for i in res:
            specs.append(Spectrum(table = pd.DataFrame(i['table']), metadata=i['metadata']))
        
        return specs

    def to_json(self, filename: Union[Path, str]) -> None:
        """
        Saves Spectrum mass-list to JSON
        
        Parameters
        ----------
        filename: str
            Path for saving mass spectrum table with calculation to json file
        """

        res = []
        for spec in self:
            out = {'metadata':copy.deepcopy(dict(spec.metadata))}
            out['table'] = spec.table.to_dict()
            res.append(out)

        with open(filename, 'w') as f:
            json.dump(res, f)

    def to_csv(self, folder: Union[Path, str]) -> None:
        """
        Save all spectrum into separate csv files

        Parameters
        ----------
        folder: str
            folder for save spectra in separate files
        """

        for spec in self:
            name = f'{spec.metadata["name"]}.csv'
            spec.table.to_csv(os.path.join(folder, name))

    @staticmethod
    def read_csv(folder: Union[Path, str]) -> 'SpectrumList':
        """
        Read csv files from folder to SpectrumList object. 
        Read only 'csv' ot 'txt' fromat

        Parameters
        ----------
        folder: str
            folder for save spectra in separate files

        Return
        ------
        SpectrumList
        """

        specs = SpectrumList()

        for file in os.listdir(folder):
            if file[-3:].lower() == 'txt' or file[-3:].lower() == 'csv':
                spec = Spectrum.read_csv(filename=os.path.join(folder, file))
                specs.append(spec)
        
        return specs

    def get_names(self) -> Sequence:
        """
        Get names of spectra

        Return
        ------
        List
            list with names of spectra in SpectrumList
        """
        return [spec.metadata['name'] for spec in self]

    def get_simmilarity(self, mode: str = "cosine", symmetric = True) -> np.ndarray:
        """
        Calculate simmilarity matrix for all spectra in SpectrumList

        Parameters
        ----------
        mode: {"tanimoto", "jaccard", "cosine"}
            Optionaly. Default cosine. 
            one of the simmilarity functions
            Mode can be: "tanimoto", "jaccard", "cosine"
        symmetric: bool
            Optionaly. Default True.
            If metric is symmetrical ( a(b)==b(a) ) it is enough to calc just half of table

        Return
        ------
        numpy array
            table with simmilirities of spectrum corresponig their index in SpectrumList
        """
        
        spec_num = len(self)
        values = np.eye(spec_num)

        for x in range(spec_num):
            if symmetric:
                for y in range(x+1, spec_num):
                    values[x,y] = self[x].simmilarity(self[y], mode=mode)
            else:
                for y in range(spec_num):
                    values[x,y] = self[x].simmilarity(self[y], mode=mode)
        
        if symmetric:
            values = values + values.T - np.diag(np.diag(values))

        return values

    def get_mol_metrics(self, 
                        metrics: Optional[Sequence[str]] = None,
                        func: Optional[str] = None) -> pd.DataFrame:
        """
        Get average molecular metrics

        Parameters
        ----------
        metrics: Sequence[str]
            Optional. Default None. Chose metrics fot watch.
        func: {"weight", "mean", "median", "max", "min", "std"}
            How calculate average. My be "weight" (default - weight average on intensity),
            "mean", "median", "max", "min", "std" (standard deviation)

        Return
        ------
        Pandas Dataframe 
        """

        metrics_table = pd.DataFrame()        
        names = []

        for i, spec in enumerate(self):
            metr = spec.get_mol_metrics(metrics=metrics, func=func)
            names.append(spec.metadata['name'])
            if i == 0:
                index = metr['metric'].values
            metrics_table[spec.metadata['name']] = metr['value']

        metrics_table.index = index
        metrics_table.columns = names

        return metrics_table

    def get_square_vk(self, how_average: str = 'weight') -> pd.DataFrame:
        """
        Calculate Van-Krevelen square density for spectra

        Parameters
        ----------
        how_average: {"count", "weight"}
            How calculate average. My be "count" or "weight" ((default))

        Return
        ------
        Pandas Dataframe

        References
        ----------
        Perminova I. V. From green chemistry and nature-like technologies towards 
        ecoadaptive chemistry and technology // Pure and Applied Chemistry. 
        2019. Vol. 91, № 5. P. 851-864.
        """

        square_vk = pd.DataFrame()
        for i, spec in enumerate(self):
            square_dens = spec.get_squares_vk(how_average=how_average)
            if i == 0:
                index = square_dens['square'].values
            square_vk[spec.metadata['name']] = square_dens['value']
        square_vk.index = index

        return square_vk

    def get_mol_density(self, how_average: str = "weight", how: Optional[str] = None) -> pd.DataFrame:
        """
        Calculate molecular class density table

        Parameters
        ----------
        how_average: {'weight', 'count'}
            how average density. Default "weight" - weight by intensity.
            Also can be "count".
        how: {'kellerman', 'perminova', 'laszakovits'}
            How devide to calsses. Optional. Default 'laszakovits'

        Return
        ------
        pandas Dataframe

        References
        ----------
        Laszakovits, J. R., & MacKay, A. A. Journal of the American Society for Mass Spectrometry, 2021, 33(1), 198-202.
        A. M. Kellerman, T. Dittmar, D. N. Kothawala, L. J. Tranvik. Nat. Commun. 5, 3804 (2014)
        Perminova I. V. Pure and Applied Chemistry. 2019. Vol. 91, № 5. P. 851-864
        """

        mol_density = pd.DataFrame()
        for i, spec in enumerate(self):
            mol_dens_spec = spec.get_mol_class(how_average=how_average, how=how)
            if i == 0:
                index = mol_dens_spec['class'].values
            mol_density[spec.metadata['name']] = mol_dens_spec['density']
        mol_density.index = index

        return mol_density

    def draw_mol_density(
        self,
        mol_density: Optional[pd.DataFrame] = None,
        ax: Optional[plt.axes] = None,
        **kwargs
        ) -> None:
        """
        Draw mol density of spectra in bar diagram

        Parameters
        ----------
        mol_density: pd.DataFrame
            Optional. Table with molecular class density. Default None and cacl by self.
        ax: matplotlib axes
            Entarnal axes for plot
        **kwargs: dict
            Additional parameters to seaborn heatmap method
        """
        if mol_density is None:
            mol_density = self.get_mol_density()

        if ax is None:
            fig, ax = plt.subplots(figsize=(4,4), dpi=75)

        width=0.35
        
        labels = mol_density.columns
        bottom = np.zeros(len(labels))
        for key in mol_density.index:
            val = [mol_density.at[key, i] for i in labels]
            ax.bar(labels, val, width, label=key, bottom=bottom)
            bottom = bottom + np.array(val)

        ax.set_xticklabels(labels, rotation=90, ha='right')
        
        handles, labels = ax.get_legend_handles_labels()
        ax.legend(reversed(handles), reversed(labels), loc='center left', bbox_to_anchor=(1, 0.5))

    def draw_simmilarity(
        self,
        mode: str = "cosine",
        values: Optional[np.ndarray] = None,
        ax: Optional[plt.axes] = None,
        annot: bool = True,
        **kwargs
        ) -> None:
        """
        Draw simmilarity matrix by using seaborn

        Parameters
        ----------
        values: np.ndarray
            Optionaly. simmilarity matix.
            Default None - It is call calculate_simmilarity() method.
        mode: str
            Optionaly. If values is none for calculate matrix. 
            Default cosine. one of the simmilarity functions
            Mode can be: "tanimoto", "jaccard", "cosine"
        ax: matplotlib axes
            Entarnal axes for plot
        annotate: bool
            Draw value of simmilarity onto titles
        **kwargs: dict
            Additional parameters to seaborn heatmap method
        """
        if values is None:
            values = self.get_simmilarity(mode=mode)

        if ax is None:
            fig, ax = plt.subplots(figsize=(len(self),len(self)), dpi=75)
        
        axis_labels = []
        for i, spec in enumerate(self):
            axis_labels.append(spec.metadata['name'] if 'name' in spec.metadata else i)
        
        sns.heatmap(np.array(values), vmin=0, vmax=1, cmap="viridis", annot=annot, ax=ax, xticklabels=axis_labels, yticklabels=axis_labels)
        plt.title(mode)


if __name__ == '__main__':
    pass


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gui_design.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(767, 740)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.main = QtWidgets.QWidget()
        self.main.setObjectName("main")
        self.label_14 = QtWidgets.QLabel(self.main)
        self.label_14.setGeometry(QtCore.QRect(640, 220, 60, 16))
        self.label_14.setText("")
        self.label_14.setObjectName("label_14")
        self.layoutWidget = QtWidgets.QWidget(self.main)
        self.layoutWidget.setGeometry(QtCore.QRect(10, 10, 709, 476))
        self.layoutWidget.setObjectName("layoutWidget")
        self.horizontalLayout_32 = QtWidgets.QHBoxLayout(self.layoutWidget)
        self.horizontalLayout_32.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout_32.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_32.setSpacing(3)
        self.horizontalLayout_32.setObjectName("horizontalLayout_32")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.load_spectrum = QtWidgets.QPushButton(self.layoutWidget)
        self.load_spectrum.setObjectName("load_spectrum")
        self.horizontalLayout_2.addWidget(self.load_spectrum)
        self.load_new = QtWidgets.QCheckBox(self.layoutWidget)
        self.load_new.setObjectName("load_new")
        self.horizontalLayout_2.addWidget(self.load_new)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.line_13 = QtWidgets.QFrame(self.layoutWidget)
        self.line_13.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_13.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_13.setObjectName("line_13")
        self.verticalLayout.addWidget(self.line_13)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.layoutWidget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.load_intensity = QtWidgets.QLineEdit(self.layoutWidget)
        self.load_intensity.setObjectName("load_intensity")
        self.gridLayout.addWidget(self.load_intensity, 2, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.layoutWidget)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.layoutWidget)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 3, 0, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.layoutWidget)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 4, 0, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.layoutWidget)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 5, 0, 1, 1)
        self.load_max_intens = QtWidgets.QLineEdit(self.layoutWidget)
        self.load_max_intens.setObjectName("load_max_intens")
        self.gridLayout.addWidget(self.load_max_intens, 4, 1, 1, 1)
        self.load_min_intens = QtWidgets.QLineEdit(self.layoutWidget)
        self.load_min_intens.setObjectName("load_min_intens")
        self.gridLayout.addWidget(self.load_min_intens, 3, 1, 1, 1)
        self.load_mz = QtWidgets.QLineEdit(self.layoutWidget)
        self.load_mz.setObjectName("load_mz")
        self.gridLayout.addWidget(self.load_mz, 1, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.layoutWidget)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.load_sep = QtWidgets.QLineEdit(self.layoutWidget)
        self.load_sep.setObjectName("load_sep")
        self.gridLayout.addWidget(self.load_sep, 0, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.layoutWidget)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 6, 0, 1, 1)
        self.load_max_mz = QtWidgets.QLineEdit(self.layoutWidget)
        self.load_max_mz.setObjectName("load_max_mz")
        self.gridLayout.addWidget(self.load_max_mz, 6, 1, 1, 1)
        self.load_min_mz = QtWidgets.QLineEdit(self.layoutWidget)
        self.load_min_mz.setObjectName("load_min_mz")
        self.gridLayout.addWidget(self.load_min_mz, 5, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.line_11 = QtWidgets.QFrame(self.layoutWidget)
        self.line_11.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_11.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_11.setObjectName("line_11")
        self.verticalLayout.addWidget(self.line_11)
        self.save_spectrum = QtWidgets.QPushButton(self.layoutWidget)
        self.save_spectrum.setObjectName("save_spectrum")
        self.verticalLayout.addWidget(self.save_spectrum)
        self.print_spec = QtWidgets.QPushButton(self.layoutWidget)
        self.print_spec.setObjectName("print_spec")
        self.verticalLayout.addWidget(self.print_spec)
        self.line_12 = QtWidgets.QFrame(self.layoutWidget)
        self.line_12.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_12.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_12.setObjectName("line_12")
        self.verticalLayout.addWidget(self.line_12)
        self.plot_spectrum = QtWidgets.QPushButton(self.layoutWidget)
        self.plot_spectrum.setObjectName("plot_spectrum")
        self.verticalLayout.addWidget(self.plot_spectrum)
        self.plot_vk = QtWidgets.QPushButton(self.layoutWidget)
        self.plot_vk.setObjectName("plot_vk")
        self.verticalLayout.addWidget(self.plot_vk)
        self.horizontalLayout_32.addLayout(self.verticalLayout)
        self.line = QtWidgets.QFrame(self.layoutWidget)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_32.addWidget(self.line)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.assign = QtWidgets.QPushButton(self.layoutWidget)
        self.assign.setObjectName("assign")
        self.verticalLayout_3.addWidget(self.assign)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_10 = QtWidgets.QLabel(self.layoutWidget)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_3.addWidget(self.label_10)
        self.assign_error_ppm = QtWidgets.QLineEdit(self.layoutWidget)
        self.assign_error_ppm.setObjectName("assign_error_ppm")
        self.horizontalLayout_3.addWidget(self.assign_error_ppm)
        self.mode = QtWidgets.QComboBox(self.layoutWidget)
        self.mode.setObjectName("mode")
        self.mode.addItem("")
        self.mode.addItem("")
        self.horizontalLayout_3.addWidget(self.mode)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_22 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_22.setObjectName("horizontalLayout_22")
        self.show_assign_error = QtWidgets.QPushButton(self.layoutWidget)
        self.show_assign_error.setObjectName("show_assign_error")
        self.horizontalLayout_22.addWidget(self.show_assign_error)
        self.remove_C_13 = QtWidgets.QPushButton(self.layoutWidget)
        self.remove_C_13.setObjectName("remove_C_13")
        self.horizontalLayout_22.addWidget(self.remove_C_13)
        self.duplicates = QtWidgets.QPushButton(self.layoutWidget)
        self.duplicates.setObjectName("duplicates")
        self.horizontalLayout_22.addWidget(self.duplicates)
        self.verticalLayout_3.addLayout(self.horizontalLayout_22)
        self.line_9 = QtWidgets.QFrame(self.layoutWidget)
        self.line_9.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_9.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_9.setObjectName("line_9")
        self.verticalLayout_3.addWidget(self.line_9)
        self.listWidget_2 = QtWidgets.QListWidget(self.layoutWidget)
        self.listWidget_2.setObjectName("listWidget_2")
        self.verticalLayout_3.addWidget(self.listWidget_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_8 = QtWidgets.QLabel(self.layoutWidget)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout.addWidget(self.label_8)
        self.assign_element = QtWidgets.QLineEdit(self.layoutWidget)
        self.assign_element.setObjectName("assign_element")
        self.horizontalLayout.addWidget(self.assign_element)
        self.label_9 = QtWidgets.QLabel(self.layoutWidget)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout.addWidget(self.label_9)
        self.assign_isotope = QtWidgets.QLineEdit(self.layoutWidget)
        self.assign_isotope.setObjectName("assign_isotope")
        self.horizontalLayout.addWidget(self.assign_isotope)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_12 = QtWidgets.QLabel(self.layoutWidget)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_4.addWidget(self.label_12)
        self.assign_range_min = QtWidgets.QLineEdit(self.layoutWidget)
        self.assign_range_min.setObjectName("assign_range_min")
        self.horizontalLayout_4.addWidget(self.assign_range_min)
        self.assign_range_max = QtWidgets.QLineEdit(self.layoutWidget)
        self.assign_range_max.setObjectName("assign_range_max")
        self.horizontalLayout_4.addWidget(self.assign_range_max)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.add_element = QtWidgets.QPushButton(self.layoutWidget)
        self.add_element.setObjectName("add_element")
        self.horizontalLayout_5.addWidget(self.add_element)
        self.remove_element = QtWidgets.QPushButton(self.layoutWidget)
        self.remove_element.setObjectName("remove_element")
        self.horizontalLayout_5.addWidget(self.remove_element)
        self.reset_element = QtWidgets.QPushButton(self.layoutWidget)
        self.reset_element.setObjectName("reset_element")
        self.horizontalLayout_5.addWidget(self.reset_element)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.generate_gdf = QtWidgets.QPushButton(self.layoutWidget)
        self.generate_gdf.setObjectName("generate_gdf")
        self.horizontalLayout_19.addWidget(self.generate_gdf)
        self.rules = QtWidgets.QCheckBox(self.layoutWidget)
        self.rules.setObjectName("rules")
        self.horizontalLayout_19.addWidget(self.rules)
        self.verticalLayout_3.addLayout(self.horizontalLayout_19)
        self.line_10 = QtWidgets.QFrame(self.layoutWidget)
        self.line_10.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_10.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_10.setObjectName("line_10")
        self.verticalLayout_3.addWidget(self.line_10)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.gen_tmds = QtWidgets.QPushButton(self.layoutWidget)
        self.gen_tmds.setObjectName("gen_tmds")
        self.horizontalLayout_9.addWidget(self.gen_tmds)
        self.assign_tmds = QtWidgets.QPushButton(self.layoutWidget)
        self.assign_tmds.setObjectName("assign_tmds")
        self.horizontalLayout_9.addWidget(self.assign_tmds)
        self.verticalLayout_3.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_13 = QtWidgets.QLabel(self.layoutWidget)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_6.addWidget(self.label_13)
        self.tmds_p = QtWidgets.QLineEdit(self.layoutWidget)
        self.tmds_p.setObjectName("tmds_p")
        self.horizontalLayout_6.addWidget(self.tmds_p)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_16 = QtWidgets.QLabel(self.layoutWidget)
        self.label_16.setObjectName("label_16")
        self.horizontalLayout_7.addWidget(self.label_16)
        self.max_tmds = QtWidgets.QLineEdit(self.layoutWidget)
        self.max_tmds.setObjectName("max_tmds")
        self.horizontalLayout_7.addWidget(self.max_tmds)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_7)
        self.tmds_c13 = QtWidgets.QCheckBox(self.layoutWidget)
        self.tmds_c13.setObjectName("tmds_c13")
        self.horizontalLayout_8.addWidget(self.tmds_c13)
        self.verticalLayout_3.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_32.addLayout(self.verticalLayout_3)
        self.line_2 = QtWidgets.QFrame(self.layoutWidget)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_32.addWidget(self.line_2)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.recallibrate = QtWidgets.QPushButton(self.layoutWidget)
        self.recallibrate.setObjectName("recallibrate")
        self.verticalLayout_5.addWidget(self.recallibrate)
        self.load_etalon = QtWidgets.QPushButton(self.layoutWidget)
        self.load_etalon.setObjectName("load_etalon")
        self.verticalLayout_5.addWidget(self.load_etalon)
        self.calc_recal_by_etalon = QtWidgets.QPushButton(self.layoutWidget)
        self.calc_recal_by_etalon.setObjectName("calc_recal_by_etalon")
        self.verticalLayout_5.addWidget(self.calc_recal_by_etalon)
        self.calc_self_recal_by_assign = QtWidgets.QPushButton(self.layoutWidget)
        self.calc_self_recal_by_assign.setObjectName("calc_self_recal_by_assign")
        self.verticalLayout_5.addWidget(self.calc_self_recal_by_assign)
        self.calc_self_recal_by_mdiff = QtWidgets.QPushButton(self.layoutWidget)
        self.calc_self_recal_by_mdiff.setObjectName("calc_self_recal_by_mdiff")
        self.verticalLayout_5.addWidget(self.calc_self_recal_by_mdiff)
        self.line_5 = QtWidgets.QFrame(self.layoutWidget)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.verticalLayout_5.addWidget(self.line_5)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.load_error = QtWidgets.QPushButton(self.layoutWidget)
        self.load_error.setObjectName("load_error")
        self.horizontalLayout_11.addWidget(self.load_error)
        self.save_error = QtWidgets.QPushButton(self.layoutWidget)
        self.save_error.setObjectName("save_error")
        self.horizontalLayout_11.addWidget(self.save_error)
        self.verticalLayout_4.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.show_error = QtWidgets.QPushButton(self.layoutWidget)
        self.show_error.setObjectName("show_error")
        self.horizontalLayout_10.addWidget(self.show_error)
        self.err_extra = QtWidgets.QPushButton(self.layoutWidget)
        self.err_extra.setObjectName("err_extra")
        self.horizontalLayout_10.addWidget(self.err_extra)
        self.verticalLayout_4.addLayout(self.horizontalLayout_10)
        self.verticalLayout_5.addLayout(self.verticalLayout_4)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.range = QtWidgets.QPushButton(self.layoutWidget)
        self.range.setObjectName("range")
        self.horizontalLayout_12.addWidget(self.range)
        self.recal_range_min = QtWidgets.QLineEdit(self.layoutWidget)
        self.recal_range_min.setObjectName("recal_range_min")
        self.horizontalLayout_12.addWidget(self.recal_range_min)
        self.recal_range_max = QtWidgets.QLineEdit(self.layoutWidget)
        self.recal_range_max.setObjectName("recal_range_max")
        self.horizontalLayout_12.addWidget(self.recal_range_max)
        self.verticalLayout_5.addLayout(self.horizontalLayout_12)
        self.line_4 = QtWidgets.QFrame(self.layoutWidget)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.verticalLayout_5.addWidget(self.line_4)
        self.load_background = QtWidgets.QPushButton(self.layoutWidget)
        self.load_background.setObjectName("load_background")
        self.verticalLayout_5.addWidget(self.load_background)
        self.remove_background = QtWidgets.QPushButton(self.layoutWidget)
        self.remove_background.setObjectName("remove_background")
        self.verticalLayout_5.addWidget(self.remove_background)
        self.horizontalLayout_32.addLayout(self.verticalLayout_5)
        self.tabWidget.addTab(self.main, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.layoutWidget1 = QtWidgets.QWidget(self.tab)
        self.layoutWidget1.setGeometry(QtCore.QRect(10, 4, 725, 481))
        self.layoutWidget1.setObjectName("layoutWidget1")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.layoutWidget1)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setSpacing(5)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setSpacing(4)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.multi_load = QtWidgets.QPushButton(self.layoutWidget1)
        self.multi_load.setObjectName("multi_load")
        self.horizontalLayout_15.addWidget(self.multi_load)
        self.add = QtWidgets.QPushButton(self.layoutWidget1)
        self.add.setObjectName("add")
        self.horizontalLayout_15.addWidget(self.add)
        self.verticalLayout_6.addLayout(self.horizontalLayout_15)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.save_spectrum_2 = QtWidgets.QPushButton(self.layoutWidget1)
        self.save_spectrum_2.setObjectName("save_spectrum_2")
        self.horizontalLayout_16.addWidget(self.save_spectrum_2)
        self.remove = QtWidgets.QPushButton(self.layoutWidget1)
        self.remove.setObjectName("remove")
        self.horizontalLayout_16.addWidget(self.remove)
        self.verticalLayout_6.addLayout(self.horizontalLayout_16)
        self.line_19 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_19.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_19.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_19.setObjectName("line_19")
        self.verticalLayout_6.addWidget(self.line_19)
        self.horizontalLayout_33 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_33.setObjectName("horizontalLayout_33")
        self.add_2 = QtWidgets.QPushButton(self.layoutWidget1)
        self.add_2.setObjectName("add_2")
        self.horizontalLayout_33.addWidget(self.add_2)
        self.sub = QtWidgets.QPushButton(self.layoutWidget1)
        self.sub.setObjectName("sub")
        self.horizontalLayout_33.addWidget(self.sub)
        self.and_ = QtWidgets.QPushButton(self.layoutWidget1)
        self.and_.setObjectName("and_")
        self.horizontalLayout_33.addWidget(self.and_)
        self.xor = QtWidgets.QPushButton(self.layoutWidget1)
        self.xor.setObjectName("xor")
        self.horizontalLayout_33.addWidget(self.xor)
        self.verticalLayout_6.addLayout(self.horizontalLayout_33)
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.int_sub = QtWidgets.QPushButton(self.layoutWidget1)
        self.int_sub.setObjectName("int_sub")
        self.horizontalLayout_18.addWidget(self.int_sub)
        self.venn = QtWidgets.QPushButton(self.layoutWidget1)
        self.venn.setObjectName("venn")
        self.horizontalLayout_18.addWidget(self.venn)
        self.verticalLayout_6.addLayout(self.horizontalLayout_18)
        self.line_3 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_6.addWidget(self.line_3)
        self.horizontalLayout_31 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_31.setObjectName("horizontalLayout_31")
        self.check_all = QtWidgets.QPushButton(self.layoutWidget1)
        self.check_all.setObjectName("check_all")
        self.horizontalLayout_31.addWidget(self.check_all)
        self.uncheck_all = QtWidgets.QPushButton(self.layoutWidget1)
        self.uncheck_all.setObjectName("uncheck_all")
        self.horizontalLayout_31.addWidget(self.uncheck_all)
        self.up = QtWidgets.QPushButton(self.layoutWidget1)
        self.up.setObjectName("up")
        self.horizontalLayout_31.addWidget(self.up)
        self.down = QtWidgets.QPushButton(self.layoutWidget1)
        self.down.setObjectName("down")
        self.horizontalLayout_31.addWidget(self.down)
        self.verticalLayout_6.addLayout(self.horizontalLayout_31)
        self.verticalLayout_8.addLayout(self.verticalLayout_6)
        self.listWidget = QtWidgets.QListWidget(self.layoutWidget1)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout_8.addWidget(self.listWidget)
        self.verticalLayout_8.setStretch(0, 1)
        self.verticalLayout_8.setStretch(1, 3)
        self.gridLayout_2.addLayout(self.verticalLayout_8, 0, 1, 1, 1)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setSpacing(5)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.horizontalLayout_26 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_26.setObjectName("horizontalLayout_26")
        self.calc_all = QtWidgets.QPushButton(self.layoutWidget1)
        self.calc_all.setObjectName("calc_all")
        self.horizontalLayout_26.addWidget(self.calc_all)
        self.normalize = QtWidgets.QPushButton(self.layoutWidget1)
        self.normalize.setObjectName("normalize")
        self.horizontalLayout_26.addWidget(self.normalize)
        self.nomalize_box = QtWidgets.QComboBox(self.layoutWidget1)
        self.nomalize_box.setObjectName("nomalize_box")
        self.nomalize_box.addItem("")
        self.nomalize_box.addItem("")
        self.nomalize_box.addItem("")
        self.nomalize_box.addItem("")
        self.horizontalLayout_26.addWidget(self.nomalize_box)
        self.verticalLayout_7.addLayout(self.horizontalLayout_26)
        self.line_16 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_16.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_16.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_16.setObjectName("line_16")
        self.verticalLayout_7.addWidget(self.line_16)
        self.horizontalLayout_25 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_25.setObjectName("horizontalLayout_25")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.count = QtWidgets.QPushButton(self.layoutWidget1)
        self.count.setObjectName("count")
        self.verticalLayout_9.addWidget(self.count)
        self.func = QtWidgets.QComboBox(self.layoutWidget1)
        self.func.setObjectName("func")
        self.func.addItem("")
        self.func.addItem("")
        self.func.addItem("")
        self.func.addItem("")
        self.func.addItem("")
        self.func.addItem("")
        self.verticalLayout_9.addWidget(self.func)
        self.horizontalLayout_25.addLayout(self.verticalLayout_9)
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.cut_box_2 = QtWidgets.QComboBox(self.layoutWidget1)
        self.cut_box_2.setObjectName("cut_box_2")
        self.cut_box_2.addItem("")
        self.cut_box_2.addItem("")
        self.cut_box_2.addItem("")
        self.verticalLayout_10.addWidget(self.cut_box_2)
        self.cut_box = QtWidgets.QComboBox(self.layoutWidget1)
        self.cut_box.setObjectName("cut_box")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.cut_box.addItem("")
        self.verticalLayout_10.addWidget(self.cut_box)
        self.horizontalLayout_25.addLayout(self.verticalLayout_10)
        self.cut_line = QtWidgets.QLineEdit(self.layoutWidget1)
        self.cut_line.setObjectName("cut_line")
        self.horizontalLayout_25.addWidget(self.cut_line)
        self.cut = QtWidgets.QPushButton(self.layoutWidget1)
        self.cut.setObjectName("cut")
        self.horizontalLayout_25.addWidget(self.cut)
        self.verticalLayout_7.addLayout(self.horizontalLayout_25)
        self.line_7 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_7.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_7.setObjectName("line_7")
        self.verticalLayout_7.addWidget(self.line_7)
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.classes = QtWidgets.QPushButton(self.layoutWidget1)
        self.classes.setObjectName("classes")
        self.horizontalLayout_17.addWidget(self.classes)
        self.classes_choise = QtWidgets.QComboBox(self.layoutWidget1)
        self.classes_choise.setObjectName("classes_choise")
        self.classes_choise.addItem("")
        self.classes_choise.addItem("")
        self.classes_choise.addItem("")
        self.horizontalLayout_17.addWidget(self.classes_choise)
        self.line_17 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_17.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_17.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_17.setObjectName("line_17")
        self.horizontalLayout_17.addWidget(self.line_17)
        self.plot_dbe = QtWidgets.QPushButton(self.layoutWidget1)
        self.plot_dbe.setObjectName("plot_dbe")
        self.horizontalLayout_17.addWidget(self.plot_dbe)
        self.label_18 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_17.addWidget(self.label_18)
        self.no_min = QtWidgets.QLineEdit(self.layoutWidget1)
        self.no_min.setObjectName("no_min")
        self.horizontalLayout_17.addWidget(self.no_min)
        self.no_max = QtWidgets.QLineEdit(self.layoutWidget1)
        self.no_max.setObjectName("no_max")
        self.horizontalLayout_17.addWidget(self.no_max)
        self.verticalLayout_7.addLayout(self.horizontalLayout_17)
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        self.plot_matrix = QtWidgets.QPushButton(self.layoutWidget1)
        self.plot_matrix.setObjectName("plot_matrix")
        self.horizontalLayout_21.addWidget(self.plot_matrix)
        self.similarity_metric = QtWidgets.QComboBox(self.layoutWidget1)
        self.similarity_metric.setObjectName("similarity_metric")
        self.similarity_metric.addItem("")
        self.similarity_metric.addItem("")
        self.similarity_metric.addItem("")
        self.horizontalLayout_21.addWidget(self.similarity_metric)
        self.line_18 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_18.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_18.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_18.setObjectName("line_18")
        self.horizontalLayout_21.addWidget(self.line_18)
        self.square_density_plot = QtWidgets.QPushButton(self.layoutWidget1)
        self.square_density_plot.setObjectName("square_density_plot")
        self.horizontalLayout_21.addWidget(self.square_density_plot)
        self.save_csv = QtWidgets.QPushButton(self.layoutWidget1)
        self.save_csv.setObjectName("save_csv")
        self.horizontalLayout_21.addWidget(self.save_csv)
        self.verticalLayout_7.addLayout(self.horizontalLayout_21)
        self.line_8 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_8.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_8.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_8.setObjectName("line_8")
        self.verticalLayout_7.addWidget(self.line_8)
        self.horizontalLayout_20 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_20.setObjectName("horizontalLayout_20")
        self.plot_spectrum_2 = QtWidgets.QPushButton(self.layoutWidget1)
        self.plot_spectrum_2.setObjectName("plot_spectrum_2")
        self.horizontalLayout_20.addWidget(self.plot_spectrum_2)
        self.line_15 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_15.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_15.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_15.setObjectName("line_15")
        self.horizontalLayout_20.addWidget(self.line_15)
        self.label_23 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_23.setObjectName("label_23")
        self.horizontalLayout_20.addWidget(self.label_23)
        self.size_volume = QtWidgets.QLineEdit(self.layoutWidget1)
        self.size_volume.setObjectName("size_volume")
        self.horizontalLayout_20.addWidget(self.size_volume)
        self.label_11 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_20.addWidget(self.label_11)
        self.pow = QtWidgets.QLineEdit(self.layoutWidget1)
        self.pow.setObjectName("pow")
        self.horizontalLayout_20.addWidget(self.pow)
        self.verticalLayout_7.addLayout(self.horizontalLayout_20)
        self.horizontalLayout_28 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_28.setObjectName("horizontalLayout_28")
        self.scatter = QtWidgets.QPushButton(self.layoutWidget1)
        self.scatter.setObjectName("scatter")
        self.horizontalLayout_28.addWidget(self.scatter)
        self.scatter_box_1 = QtWidgets.QComboBox(self.layoutWidget1)
        self.scatter_box_1.setObjectName("scatter_box_1")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.scatter_box_1.addItem("")
        self.horizontalLayout_28.addWidget(self.scatter_box_1)
        self.sactter_box_2 = QtWidgets.QComboBox(self.layoutWidget1)
        self.sactter_box_2.setObjectName("sactter_box_2")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.sactter_box_2.addItem("")
        self.horizontalLayout_28.addWidget(self.sactter_box_2)
        self.sactter_box_3 = QtWidgets.QComboBox(self.layoutWidget1)
        self.sactter_box_3.setObjectName("sactter_box_3")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.sactter_box_3.addItem("")
        self.horizontalLayout_28.addWidget(self.sactter_box_3)
        self.verticalLayout_7.addLayout(self.horizontalLayout_28)
        self.horizontalLayout_29 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_29.setObjectName("horizontalLayout_29")
        self.density_plot = QtWidgets.QPushButton(self.layoutWidget1)
        self.density_plot.setObjectName("density_plot")
        self.horizontalLayout_29.addWidget(self.density_plot)
        self.density = QtWidgets.QPushButton(self.layoutWidget1)
        self.density.setObjectName("density")
        self.horizontalLayout_29.addWidget(self.density)
        self.density_box = QtWidgets.QComboBox(self.layoutWidget1)
        self.density_box.setObjectName("density_box")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.density_box.addItem("")
        self.horizontalLayout_29.addWidget(self.density_box)
        self.verticalLayout_7.addLayout(self.horizontalLayout_29)
        self.line_14 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_14.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_14.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_14.setObjectName("line_14")
        self.verticalLayout_7.addWidget(self.line_14)
        self.line_6 = QtWidgets.QFrame(self.layoutWidget1)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.verticalLayout_7.addWidget(self.line_6)
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_24 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_24.setObjectName("label_24")
        self.horizontalLayout_14.addWidget(self.label_24)
        self.x1 = QtWidgets.QLineEdit(self.layoutWidget1)
        self.x1.setObjectName("x1")
        self.horizontalLayout_14.addWidget(self.x1)
        self.label_17 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_17.setObjectName("label_17")
        self.horizontalLayout_14.addWidget(self.label_17)
        self.x2 = QtWidgets.QLineEdit(self.layoutWidget1)
        self.x2.setObjectName("x2")
        self.horizontalLayout_14.addWidget(self.x2)
        self.label_25 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_25.setObjectName("label_25")
        self.horizontalLayout_14.addWidget(self.label_25)
        self.y1 = QtWidgets.QLineEdit(self.layoutWidget1)
        self.y1.setObjectName("y1")
        self.horizontalLayout_14.addWidget(self.y1)
        self.label_26 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_26.setObjectName("label_26")
        self.horizontalLayout_14.addWidget(self.label_26)
        self.y2 = QtWidgets.QLineEdit(self.layoutWidget1)
        self.y2.setObjectName("y2")
        self.horizontalLayout_14.addWidget(self.y2)
        self.verticalLayout_7.addLayout(self.horizontalLayout_14)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_15 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_13.addWidget(self.label_15)
        self.dpi_line = QtWidgets.QLineEdit(self.layoutWidget1)
        self.dpi_line.setObjectName("dpi_line")
        self.horizontalLayout_13.addWidget(self.dpi_line)
        self.label_21 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_21.setObjectName("label_21")
        self.horizontalLayout_13.addWidget(self.label_21)
        self.size_line = QtWidgets.QLineEdit(self.layoutWidget1)
        self.size_line.setObjectName("size_line")
        self.horizontalLayout_13.addWidget(self.size_line)
        self.size_line_2 = QtWidgets.QLineEdit(self.layoutWidget1)
        self.size_line_2.setObjectName("size_line_2")
        self.horizontalLayout_13.addWidget(self.size_line_2)
        self.verticalLayout_7.addLayout(self.horizontalLayout_13)
        self.horizontalLayout_27 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_27.setObjectName("horizontalLayout_27")
        self.path = QtWidgets.QPushButton(self.layoutWidget1)
        self.path.setObjectName("path")
        self.horizontalLayout_27.addWidget(self.path)
        self.format = QtWidgets.QComboBox(self.layoutWidget1)
        self.format.setObjectName("format")
        self.format.addItem("")
        self.format.addItem("")
        self.format.addItem("")
        self.format.addItem("")
        self.horizontalLayout_27.addWidget(self.format)
        self.save_box = QtWidgets.QCheckBox(self.layoutWidget1)
        self.save_box.setObjectName("save_box")
        self.horizontalLayout_27.addWidget(self.save_box)
        self.verticalLayout_7.addLayout(self.horizontalLayout_27)
        self.gridLayout_2.addLayout(self.verticalLayout_7, 0, 2, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.verticalLayout_2.addWidget(self.tabWidget)
        self.textBrowser = QtWidgets.QTextBrowser(self.centralwidget)
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout_2.addWidget(self.textBrowser)
        self.verticalLayout_2.setStretch(0, 3)
        self.verticalLayout_2.setStretch(1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.tabWidget, self.load_spectrum)
        MainWindow.setTabOrder(self.load_spectrum, self.load_new)
        MainWindow.setTabOrder(self.load_new, self.load_sep)
        MainWindow.setTabOrder(self.load_sep, self.load_mz)
        MainWindow.setTabOrder(self.load_mz, self.load_intensity)
        MainWindow.setTabOrder(self.load_intensity, self.load_min_intens)
        MainWindow.setTabOrder(self.load_min_intens, self.load_max_intens)
        MainWindow.setTabOrder(self.load_max_intens, self.load_min_mz)
        MainWindow.setTabOrder(self.load_min_mz, self.load_max_mz)
        MainWindow.setTabOrder(self.load_max_mz, self.save_spectrum)
        MainWindow.setTabOrder(self.save_spectrum, self.print_spec)
        MainWindow.setTabOrder(self.print_spec, self.plot_spectrum)
        MainWindow.setTabOrder(self.plot_spectrum, self.plot_vk)
        MainWindow.setTabOrder(self.plot_vk, self.assign)
        MainWindow.setTabOrder(self.assign, self.assign_error_ppm)
        MainWindow.setTabOrder(self.assign_error_ppm, self.mode)
        MainWindow.setTabOrder(self.mode, self.show_assign_error)
        MainWindow.setTabOrder(self.show_assign_error, self.remove_C_13)
        MainWindow.setTabOrder(self.remove_C_13, self.duplicates)
        MainWindow.setTabOrder(self.duplicates, self.listWidget_2)
        MainWindow.setTabOrder(self.listWidget_2, self.assign_element)
        MainWindow.setTabOrder(self.assign_element, self.assign_isotope)
        MainWindow.setTabOrder(self.assign_isotope, self.assign_range_min)
        MainWindow.setTabOrder(self.assign_range_min, self.assign_range_max)
        MainWindow.setTabOrder(self.assign_range_max, self.add_element)
        MainWindow.setTabOrder(self.add_element, self.remove_element)
        MainWindow.setTabOrder(self.remove_element, self.reset_element)
        MainWindow.setTabOrder(self.reset_element, self.generate_gdf)
        MainWindow.setTabOrder(self.generate_gdf, self.rules)
        MainWindow.setTabOrder(self.rules, self.gen_tmds)
        MainWindow.setTabOrder(self.gen_tmds, self.assign_tmds)
        MainWindow.setTabOrder(self.assign_tmds, self.tmds_p)
        MainWindow.setTabOrder(self.tmds_p, self.max_tmds)
        MainWindow.setTabOrder(self.max_tmds, self.tmds_c13)
        MainWindow.setTabOrder(self.tmds_c13, self.recallibrate)
        MainWindow.setTabOrder(self.recallibrate, self.load_etalon)
        MainWindow.setTabOrder(self.load_etalon, self.calc_recal_by_etalon)
        MainWindow.setTabOrder(self.calc_recal_by_etalon, self.calc_self_recal_by_assign)
        MainWindow.setTabOrder(self.calc_self_recal_by_assign, self.calc_self_recal_by_mdiff)
        MainWindow.setTabOrder(self.calc_self_recal_by_mdiff, self.load_error)
        MainWindow.setTabOrder(self.load_error, self.save_error)
        MainWindow.setTabOrder(self.save_error, self.show_error)
        MainWindow.setTabOrder(self.show_error, self.err_extra)
        MainWindow.setTabOrder(self.err_extra, self.range)
        MainWindow.setTabOrder(self.range, self.recal_range_min)
        MainWindow.setTabOrder(self.recal_range_min, self.recal_range_max)
        MainWindow.setTabOrder(self.recal_range_max, self.load_background)
        MainWindow.setTabOrder(self.load_background, self.remove_background)
        MainWindow.setTabOrder(self.remove_background, self.multi_load)
        MainWindow.setTabOrder(self.multi_load, self.add)
        MainWindow.setTabOrder(self.add, self.save_spectrum_2)
        MainWindow.setTabOrder(self.save_spectrum_2, self.remove)
        MainWindow.setTabOrder(self.remove, self.add_2)
        MainWindow.setTabOrder(self.add_2, self.sub)
        MainWindow.setTabOrder(self.sub, self.and_)
        MainWindow.setTabOrder(self.and_, self.xor)
        MainWindow.setTabOrder(self.xor, self.int_sub)
        MainWindow.setTabOrder(self.int_sub, self.venn)
        MainWindow.setTabOrder(self.venn, self.check_all)
        MainWindow.setTabOrder(self.check_all, self.uncheck_all)
        MainWindow.setTabOrder(self.uncheck_all, self.up)
        MainWindow.setTabOrder(self.up, self.down)
        MainWindow.setTabOrder(self.down, self.listWidget)
        MainWindow.setTabOrder(self.listWidget, self.calc_all)
        MainWindow.setTabOrder(self.calc_all, self.normalize)
        MainWindow.setTabOrder(self.normalize, self.nomalize_box)
        MainWindow.setTabOrder(self.nomalize_box, self.count)
        MainWindow.setTabOrder(self.count, self.cut_box_2)
        MainWindow.setTabOrder(self.cut_box_2, self.func)
        MainWindow.setTabOrder(self.func, self.cut_box)
        MainWindow.setTabOrder(self.cut_box, self.cut_line)
        MainWindow.setTabOrder(self.cut_line, self.cut)
        MainWindow.setTabOrder(self.cut, self.classes)
        MainWindow.setTabOrder(self.classes, self.classes_choise)
        MainWindow.setTabOrder(self.classes_choise, self.plot_dbe)
        MainWindow.setTabOrder(self.plot_dbe, self.no_min)
        MainWindow.setTabOrder(self.no_min, self.no_max)
        MainWindow.setTabOrder(self.no_max, self.plot_matrix)
        MainWindow.setTabOrder(self.plot_matrix, self.similarity_metric)
        MainWindow.setTabOrder(self.similarity_metric, self.square_density_plot)
        MainWindow.setTabOrder(self.square_density_plot, self.save_csv)
        MainWindow.setTabOrder(self.save_csv, self.plot_spectrum_2)
        MainWindow.setTabOrder(self.plot_spectrum_2, self.size_volume)
        MainWindow.setTabOrder(self.size_volume, self.pow)
        MainWindow.setTabOrder(self.pow, self.scatter)
        MainWindow.setTabOrder(self.scatter, self.scatter_box_1)
        MainWindow.setTabOrder(self.scatter_box_1, self.sactter_box_2)
        MainWindow.setTabOrder(self.sactter_box_2, self.sactter_box_3)
        MainWindow.setTabOrder(self.sactter_box_3, self.density_plot)
        MainWindow.setTabOrder(self.density_plot, self.density)
        MainWindow.setTabOrder(self.density, self.density_box)
        MainWindow.setTabOrder(self.density_box, self.x1)
        MainWindow.setTabOrder(self.x1, self.x2)
        MainWindow.setTabOrder(self.x2, self.y1)
        MainWindow.setTabOrder(self.y1, self.y2)
        MainWindow.setTabOrder(self.y2, self.dpi_line)
        MainWindow.setTabOrder(self.dpi_line, self.size_line)
        MainWindow.setTabOrder(self.size_line, self.size_line_2)
        MainWindow.setTabOrder(self.size_line_2, self.path)
        MainWindow.setTabOrder(self.path, self.format)
        MainWindow.setTabOrder(self.format, self.save_box)
        MainWindow.setTabOrder(self.save_box, self.textBrowser)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "nomspectra"))
        self.load_spectrum.setText(_translate("MainWindow", "Load Spectrum"))
        self.load_new.setText(_translate("MainWindow", "New"))
        self.label.setText(_translate("MainWindow", "Separator"))
        self.label_3.setText(_translate("MainWindow", "Intensity"))
        self.label_4.setText(_translate("MainWindow", "min Intens"))
        self.label_5.setText(_translate("MainWindow", "max Intens"))
        self.label_6.setText(_translate("MainWindow", "min m/z"))
        self.label_2.setText(_translate("MainWindow", "m/z"))
        self.label_7.setText(_translate("MainWindow", "max m/z"))
        self.save_spectrum.setText(_translate("MainWindow", "Save Spectrum"))
        self.print_spec.setText(_translate("MainWindow", "Print spectrum"))
        self.plot_spectrum.setText(_translate("MainWindow", "Plot Spectrum"))
        self.plot_vk.setText(_translate("MainWindow", "Plot Van Krevelin"))
        self.assign.setText(_translate("MainWindow", "Assign"))
        self.label_10.setText(_translate("MainWindow", "Err ppm"))
        self.mode.setItemText(0, _translate("MainWindow", "negative"))
        self.mode.setItemText(1, _translate("MainWindow", "positive"))
        self.show_assign_error.setText(_translate("MainWindow", "ShowErr"))
        self.remove_C_13.setText(_translate("MainWindow", "C_13"))
        self.duplicates.setText(_translate("MainWindow", "Dupl"))
        self.label_8.setText(_translate("MainWindow", "Element"))
        self.label_9.setText(_translate("MainWindow", "Isotope"))
        self.label_12.setText(_translate("MainWindow", "Range"))
        self.add_element.setText(_translate("MainWindow", "Add"))
        self.remove_element.setText(_translate("MainWindow", "Remove"))
        self.reset_element.setText(_translate("MainWindow", "Reset"))
        self.generate_gdf.setText(_translate("MainWindow", "Gen"))
        self.rules.setText(_translate("MainWindow", "Rules"))
        self.gen_tmds.setText(_translate("MainWindow", "Gen TMDS"))
        self.assign_tmds.setText(_translate("MainWindow", "Assign TMDS"))
        self.label_13.setText(_translate("MainWindow", "P"))
        self.label_16.setText(_translate("MainWindow", "N"))
        self.tmds_c13.setText(_translate("MainWindow", "C_13"))
        self.recallibrate.setText(_translate("MainWindow", "Recallibrate"))
        self.load_etalon.setText(_translate("MainWindow", "Load Etalon"))
        self.calc_recal_by_etalon.setText(_translate("MainWindow", "Calc Recal by Etalon"))
        self.calc_self_recal_by_assign.setText(_translate("MainWindow", "Calc Recal by Assign"))
        self.calc_self_recal_by_mdiff.setText(_translate("MainWindow", "Calc Recal by MDiff"))
        self.load_error.setText(_translate("MainWindow", "Load Error"))
        self.save_error.setText(_translate("MainWindow", "Save Error"))
        self.show_error.setText(_translate("MainWindow", "Show Error"))
        self.err_extra.setText(_translate("MainWindow", "Extrapolate"))
        self.range.setText(_translate("MainWindow", "Range"))
        self.load_background.setText(_translate("MainWindow", "Load Background"))
        self.remove_background.setText(_translate("MainWindow", "Remove Background"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.main), _translate("MainWindow", "treatment"))
        self.multi_load.setText(_translate("MainWindow", "Multi Load"))
        self.add.setText(_translate("MainWindow", "Add_Bufer"))
        self.save_spectrum_2.setText(_translate("MainWindow", "Save"))
        self.remove.setText(_translate("MainWindow", "Remove"))
        self.add_2.setText(_translate("MainWindow", "add"))
        self.sub.setText(_translate("MainWindow", "sub"))
        self.and_.setText(_translate("MainWindow", "and"))
        self.xor.setText(_translate("MainWindow", "xor"))
        self.int_sub.setText(_translate("MainWindow", "int_sub"))
        self.venn.setText(_translate("MainWindow", "venn"))
        self.check_all.setText(_translate("MainWindow", "all"))
        self.uncheck_all.setText(_translate("MainWindow", "none"))
        self.up.setText(_translate("MainWindow", "↑"))
        self.down.setText(_translate("MainWindow", "↓"))
        self.calc_all.setText(_translate("MainWindow", "Calculate"))
        self.normalize.setText(_translate("MainWindow", "Normalize"))
        self.nomalize_box.setItemText(0, _translate("MainWindow", "sum"))
        self.nomalize_box.setItemText(1, _translate("MainWindow", "max"))
        self.nomalize_box.setItemText(2, _translate("MainWindow", "mean"))
        self.nomalize_box.setItemText(3, _translate("MainWindow", "median"))
        self.count.setText(_translate("MainWindow", "Count"))
        self.func.setItemText(0, _translate("MainWindow", "weight"))
        self.func.setItemText(1, _translate("MainWindow", "mean"))
        self.func.setItemText(2, _translate("MainWindow", "median"))
        self.func.setItemText(3, _translate("MainWindow", "max"))
        self.func.setItemText(4, _translate("MainWindow", "min"))
        self.func.setItemText(5, _translate("MainWindow", "std"))
        self.cut_box_2.setItemText(0, _translate("MainWindow", ">"))
        self.cut_box_2.setItemText(1, _translate("MainWindow", "<"))
        self.cut_box_2.setItemText(2, _translate("MainWindow", "="))
        self.cut_box.setItemText(0, _translate("MainWindow", "intensity"))
        self.cut_box.setItemText(1, _translate("MainWindow", "mass"))
        self.cut_box.setItemText(2, _translate("MainWindow", "Ke"))
        self.cut_box.setItemText(3, _translate("MainWindow", "KMD"))
        self.cut_box.setItemText(4, _translate("MainWindow", "C"))
        self.cut_box.setItemText(5, _translate("MainWindow", "H"))
        self.cut_box.setItemText(6, _translate("MainWindow", "O"))
        self.cut_box.setItemText(7, _translate("MainWindow", "N"))
        self.cut_box.setItemText(8, _translate("MainWindow", "S"))
        self.cut_box.setItemText(9, _translate("MainWindow", "H/C"))
        self.cut_box.setItemText(10, _translate("MainWindow", "O/C"))
        self.cut_box.setItemText(11, _translate("MainWindow", "AI"))
        self.cut_box.setItemText(12, _translate("MainWindow", "CAI"))
        self.cut_box.setItemText(13, _translate("MainWindow", "DBE"))
        self.cut_box.setItemText(14, _translate("MainWindow", "DBE_AI"))
        self.cut_box.setItemText(15, _translate("MainWindow", "DBE-O"))
        self.cut_box.setItemText(16, _translate("MainWindow", "DBE-OC"))
        self.cut_box.setItemText(17, _translate("MainWindow", "CRAM"))
        self.cut_box.setItemText(18, _translate("MainWindow", "NOSC"))
        self.cut.setText(_translate("MainWindow", "Cut"))
        self.classes.setText(_translate("MainWindow", "Classes"))
        self.classes_choise.setItemText(0, _translate("MainWindow", "laszakovits"))
        self.classes_choise.setItemText(1, _translate("MainWindow", "kellerman"))
        self.classes_choise.setItemText(2, _translate("MainWindow", "perminova"))
        self.plot_dbe.setText(_translate("MainWindow", "DBE vs nO"))
        self.label_18.setText(_translate("MainWindow", "nO"))
        self.plot_matrix.setText(_translate("MainWindow", "Simmilarity"))
        self.similarity_metric.setItemText(0, _translate("MainWindow", "tanimoto"))
        self.similarity_metric.setItemText(1, _translate("MainWindow", "jaccard"))
        self.similarity_metric.setItemText(2, _translate("MainWindow", "cosine"))
        self.square_density_plot.setText(_translate("MainWindow", "Squares"))
        self.save_csv.setText(_translate("MainWindow", "SaveCSV"))
        self.plot_spectrum_2.setText(_translate("MainWindow", "Spectrum"))
        self.label_23.setText(_translate("MainWindow", "Size"))
        self.label_11.setText(_translate("MainWindow", "Pow"))
        self.scatter.setText(_translate("MainWindow", "Scatter"))
        self.scatter_box_1.setItemText(0, _translate("MainWindow", "O/C"))
        self.scatter_box_1.setItemText(1, _translate("MainWindow", "H/C"))
        self.scatter_box_1.setItemText(2, _translate("MainWindow", "Ke"))
        self.scatter_box_1.setItemText(3, _translate("MainWindow", "KMD"))
        self.scatter_box_1.setItemText(4, _translate("MainWindow", "intensity"))
        self.scatter_box_1.setItemText(5, _translate("MainWindow", "mass"))
        self.scatter_box_1.setItemText(6, _translate("MainWindow", "C"))
        self.scatter_box_1.setItemText(7, _translate("MainWindow", "H"))
        self.scatter_box_1.setItemText(8, _translate("MainWindow", "O"))
        self.scatter_box_1.setItemText(9, _translate("MainWindow", "N"))
        self.scatter_box_1.setItemText(10, _translate("MainWindow", "S"))
        self.scatter_box_1.setItemText(11, _translate("MainWindow", "CAI"))
        self.scatter_box_1.setItemText(12, _translate("MainWindow", "AI"))
        self.scatter_box_1.setItemText(13, _translate("MainWindow", "DBE"))
        self.scatter_box_1.setItemText(14, _translate("MainWindow", "DBE_AI"))
        self.scatter_box_1.setItemText(15, _translate("MainWindow", "DBE-O"))
        self.scatter_box_1.setItemText(16, _translate("MainWindow", "DBE-OC"))
        self.scatter_box_1.setItemText(17, _translate("MainWindow", "NOSC"))
        self.sactter_box_2.setItemText(0, _translate("MainWindow", "H/C"))
        self.sactter_box_2.setItemText(1, _translate("MainWindow", "O/C"))
        self.sactter_box_2.setItemText(2, _translate("MainWindow", "KMD"))
        self.sactter_box_2.setItemText(3, _translate("MainWindow", "Ke"))
        self.sactter_box_2.setItemText(4, _translate("MainWindow", "NOSC"))
        self.sactter_box_2.setItemText(5, _translate("MainWindow", "intensity"))
        self.sactter_box_2.setItemText(6, _translate("MainWindow", "mass"))
        self.sactter_box_2.setItemText(7, _translate("MainWindow", "C"))
        self.sactter_box_2.setItemText(8, _translate("MainWindow", "H"))
        self.sactter_box_2.setItemText(9, _translate("MainWindow", "O"))
        self.sactter_box_2.setItemText(10, _translate("MainWindow", "N"))
        self.sactter_box_2.setItemText(11, _translate("MainWindow", "S"))
        self.sactter_box_2.setItemText(12, _translate("MainWindow", "CAI"))
        self.sactter_box_2.setItemText(13, _translate("MainWindow", "AI"))
        self.sactter_box_2.setItemText(14, _translate("MainWindow", "DBE"))
        self.sactter_box_2.setItemText(15, _translate("MainWindow", "DBE_AI"))
        self.sactter_box_2.setItemText(16, _translate("MainWindow", "DBE-O"))
        self.sactter_box_2.setItemText(17, _translate("MainWindow", "DBE-OC"))
        self.sactter_box_3.setItemText(0, _translate("MainWindow", "intensity"))
        self.sactter_box_3.setItemText(1, _translate("MainWindow", "None"))
        self.sactter_box_3.setItemText(2, _translate("MainWindow", "Ke"))
        self.sactter_box_3.setItemText(3, _translate("MainWindow", "KMD"))
        self.sactter_box_3.setItemText(4, _translate("MainWindow", "mass"))
        self.sactter_box_3.setItemText(5, _translate("MainWindow", "NOSC"))
        self.sactter_box_3.setItemText(6, _translate("MainWindow", "C"))
        self.sactter_box_3.setItemText(7, _translate("MainWindow", "H"))
        self.sactter_box_3.setItemText(8, _translate("MainWindow", "O"))
        self.sactter_box_3.setItemText(9, _translate("MainWindow", "N"))
        self.sactter_box_3.setItemText(10, _translate("MainWindow", "S"))
        self.sactter_box_3.setItemText(11, _translate("MainWindow", "H/C"))
        self.sactter_box_3.setItemText(12, _translate("MainWindow", "O/C"))
        self.sactter_box_3.setItemText(13, _translate("MainWindow", "CAI"))
        self.sactter_box_3.setItemText(14, _translate("MainWindow", "AI"))
        self.sactter_box_3.setItemText(15, _translate("MainWindow", "DBE"))
        self.sactter_box_3.setItemText(16, _translate("MainWindow", "DBE_AI"))
        self.sactter_box_3.setItemText(17, _translate("MainWindow", "DBE-O"))
        self.sactter_box_3.setItemText(18, _translate("MainWindow", "DBE-OC"))
        self.density_plot.setText(_translate("MainWindow", "ScatterDens"))
        self.density.setText(_translate("MainWindow", "Density"))
        self.density_box.setItemText(0, _translate("MainWindow", "NOSC"))
        self.density_box.setItemText(1, _translate("MainWindow", "O/C"))
        self.density_box.setItemText(2, _translate("MainWindow", "H/C"))
        self.density_box.setItemText(3, _translate("MainWindow", "Ke"))
        self.density_box.setItemText(4, _translate("MainWindow", "KMD"))
        self.density_box.setItemText(5, _translate("MainWindow", "intensity"))
        self.density_box.setItemText(6, _translate("MainWindow", "mass"))
        self.density_box.setItemText(7, _translate("MainWindow", "C"))
        self.density_box.setItemText(8, _translate("MainWindow", "H"))
        self.density_box.setItemText(9, _translate("MainWindow", "O"))
        self.density_box.setItemText(10, _translate("MainWindow", "N"))
        self.density_box.setItemText(11, _translate("MainWindow", "S"))
        self.density_box.setItemText(12, _translate("MainWindow", "CAI"))
        self.density_box.setItemText(13, _translate("MainWindow", "AI"))
        self.density_box.setItemText(14, _translate("MainWindow", "DBE"))
        self.density_box.setItemText(15, _translate("MainWindow", "DBE_AI"))
        self.density_box.setItemText(16, _translate("MainWindow", "DBE-O"))
        self.density_box.setItemText(17, _translate("MainWindow", "DBE-OC"))
        self.label_24.setText(_translate("MainWindow", "x1"))
        self.label_17.setText(_translate("MainWindow", "x2"))
        self.label_25.setText(_translate("MainWindow", "y1"))
        self.label_26.setText(_translate("MainWindow", "y2"))
        self.label_15.setText(_translate("MainWindow", "dpi"))
        self.label_21.setText(_translate("MainWindow", "size"))
        self.path.setText(_translate("MainWindow", "Path"))
        self.format.setItemText(0, _translate("MainWindow", "png"))
        self.format.setItemText(1, _translate("MainWindow", "svg"))
        self.format.setItemText(2, _translate("MainWindow", "pdf"))
        self.format.setItemText(3, _translate("MainWindow", "tif"))
        self.save_box.setText(_translate("MainWindow", "Save"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("MainWindow", "analysis"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())



#    Copyright 2022 Volikov Alexander <ab.volikov@gmail.com>
#
#    nomspectra is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nomspectra is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nomspectra.  If not, see <http://www.gnu.org/licenses/>.

import copy
import traceback
import os
import sys
sys.path.append('..')

import warnings
warnings.filterwarnings('ignore')

from .gui_design import Ui_MainWindow
from .gui_dialog import Ui_Dialog
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QFileDialog, QDialog

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
from matplotlib_venn import venn2, venn3
from nomspectra.spectrum import Spectrum 
from nomspectra.spectra import SpectrumList
from nomspectra.recal import ErrorTable, recallibrate
from nomspectra.diff import Tmds, assign_by_tmds
from nomspectra.brutto import brutto_gen
import nomspectra.draw as draw

here = os.path.abspath(os.path.dirname(__file__))
version = {}
with open(os.path.join(here, "__version__.py")) as f:
    exec(f.read(), version)

about =f'''nomspectra. Version {version["__version__"]}

Graphical user interface for nomspectra package: https://github.com/nomspectra/nomspectra

Tutorial: https://nomspectra.readthedocs.io/en/latest/gui_tutorial.html

Distributed under license GPLv3 http://www.gnu.org/licenses/
'''

def_colors = ['blue','red','green','orange','purple','brown','pink','gray','olive','cyan']

class ListDialog(QtWidgets.QDialog, Ui_Dialog):
    def __init__(self, spec = None):
        super().__init__()
        self.setupUi(self)

        self.spec = spec
        
        self.Name.setText(self.spec.metadata['name'])
        self.Color.setText(self.spec.metadata['color'])
        self.Alpha.setText(str(self.spec.metadata['alpha']))

        self.OK.clicked.connect(self.ok_)
    
    def ok_(self):
        self.spec.metadata['name'] = self.Name.text()
        self.spec.metadata['color'] = self.Color.text()
        self.spec.metadata['alpha'] = float(self.Alpha.text())
        self.accept()

class App(QtWidgets.QMainWindow, Ui_MainWindow):
    def __init__(self):
        super().__init__()
        self.setupUi(self)

        self.addText(about)

        self.spec = Spectrum()
        self.back = Spectrum()
        self.etalon = Spectrum()
        self.err = ErrorTable()
        self.squares = pd.DataFrame()
        self.tmds = Tmds()
        self.elems = {'C':(4, 51),'H':(4, 101),'O':(0,26), 'N':(0,4), 'S':(0,3)}
        self.temp_element = ''
        self.gdf_elems = {}
        self.gdf = pd.DataFrame()
        self.specs_list = SpectrumList()
        self.temp_list = SpectrumList()
        self.plot_color = []
        self.plot_alpha = []
        self.path_img = None
        self.temp_name_img = 1
        self.temp_df = pd.DataFrame()

        self.listWidget.itemDoubleClicked.connect(self.list_double_clicked_)
        self.refresh_list_2()

        self.load_sep.setText('')
        self.dpi_line.setText('75')
        self.size_line.setText('4')
        self.size_line_2.setText('4')
        self.load_mz.setText('')
        self.load_intensity.setText('')
        self.load_min_intens.setText('')
        self.load_max_intens.setText('')
        self.load_min_mz.setText('')
        self.load_max_mz.setText('')
        self.assign_element.setText('')
        self.assign_isotope.setText('')
        self.assign_range_max.setText('')
        self.assign_range_min.setText('')
        self.assign_error_ppm.setText('0.5')
        self.recal_range_max.setText('')
        self.recal_range_min.setText('')
        self.tmds_p.setText('')

        self.rules.setChecked(True)
        self.tmds_c13.setChecked(True)
        self.save_box.setChecked(False)

        self.load_spectrum.clicked.connect(self.load_spectrum_)
        self.save_spectrum.clicked.connect(self.save_spectrum_)
        self.print_spec.clicked.connect(self.print_spectrum_)
        self.assign.clicked.connect(self.assign_)
        self.add_element.clicked.connect(self.add_element_)
        self.remove_element.clicked.connect(self.remove_element_)
        self.reset_element.clicked.connect(self.reset_element_)
        self.show_assign_error.clicked.connect(self.show_assign_error_)
        self.recallibrate.clicked.connect(self.recallibrate_)
        self.load_etalon.clicked.connect(self.load_etalon_)
        self.calc_recal_by_etalon.clicked.connect(self.calc_recal_by_etalon_)
        self.calc_self_recal_by_assign.clicked.connect(self.calc_self_recal_by_assign_)
        self.calc_self_recal_by_mdiff.clicked.connect(self.calc_self_recal_by_mdiff_)
        self.load_error.clicked.connect(self.load_error_)
        self.save_error.clicked.connect(self.save_error_)
        self.show_error.clicked.connect(self.show_error_)
        self.remove_C_13.clicked.connect(self.remove_C_13_)
        self.duplicates.clicked.connect(self.duplicates_)
        self.plot_spectrum.clicked.connect(self.plot_spectrum_)
        self.plot_vk.clicked.connect(self.plot_van_krevelen)
        self.range.clicked.connect(self.range_)
        self.err_extra.clicked.connect(self.extrapolate_)
        self.generate_gdf.clicked.connect(self.generate_gdf_)
        self.gen_tmds.clicked.connect(self.gen_tmds_)
        self.assign_tmds.clicked.connect(self.assign_by_tmds_)
        self.load_background.clicked.connect(self.load_background_)
        self.remove_background.clicked.connect(self.remove_background_)
        self.multi_load.clicked.connect(self.multi_load_)
        self.add.clicked.connect(self.add_bufer_)
        self.save_spectrum_2.clicked.connect(self.save_)
        self.remove.clicked.connect(self.remove_)
        self.add_2.clicked.connect(self.add_operate)
        self.sub.clicked.connect(self.sub_operate)
        self.and_.clicked.connect(self.and_operate)
        self.xor.clicked.connect(self.xor_operate)
        self.int_sub.clicked.connect(self.int_sub_operate)
        self.venn.clicked.connect(self.venn_operate)
        self.check_all.clicked.connect(self.check_all_)
        self.uncheck_all.clicked.connect(self.uncheck_all_)
        self.up.clicked.connect(self.up_)
        self.down.clicked.connect(self.down_)
        self.calc_all.clicked.connect(self.calculate_)
        self.count.clicked.connect(self.count_)
        self.normalize.clicked.connect(self.normalize_)
        self.cut.clicked.connect(self.cut_)
        self.classes.clicked.connect(self.classes_)
        self.square_density_plot.clicked.connect(self.squares_)
        self.plot_matrix.clicked.connect(self.simmilarity_)
        self.plot_dbe.clicked.connect(self.dbe_vs_no)
        self.save_csv.clicked.connect(self.save_csv_)
        self.plot_spectrum_2.clicked.connect(self.spectrum_)
        self.scatter.clicked.connect(self.scatter_)
        self.density.clicked.connect(self.density_)
        self.density_plot.clicked.connect(self.scatter_dens)
        self.path.clicked.connect(self.path_)

    def addText(self,text):

        self.textBrowser.setPlainText(text)
        
    def load_spectrum_(self):

        try:
            file, _ = QFileDialog.getOpenFileName(self, 'Open File')
            if file:
                self.addText(file)

            sep = self.load_sep.text()
            if sep == 'tab':
                sep = '\t'
            elif sep == '':
                sep = ','

            mz_head = self.load_mz.text()
            if mz_head == '':
                mz_head = 'mass'

            intens_head = self.load_intensity.text()
            if intens_head == '':
                intens_head = 'intensity'

            intens_min = self.load_min_intens.text()
            if intens_min != '':
                intens_min = int(intens_min)
            else:
                intens_min = None

            intens_max = self.load_max_intens.text()
            if intens_max != '':
                intens_max = int(intens_max)
            else:
                intens_max = None

            mz_min = self.load_min_mz.text()
            if mz_min != '':
                mz_min = int(mz_min)
            else:
                mz_min = None

            mz_max = self.load_max_mz.text()
            if mz_max != '':
                mz_max = int(mz_max)
            else:
                mz_max = None

            if self.load_new.isChecked():
                new = True
            else:
                new = False

            if file != "":
                self.spec = Spectrum().read_csv(filename=file,
                                                mapper={mz_head:'mass', intens_head:'intensity'},
                                                take_only_mz=new,
                                                sep=sep,
                                                intens_min=intens_min,
                                                intens_max=intens_max,
                                                mass_min=mz_min,
                                                mass_max=mz_max
                                                )

        except Exception:
            self.addText(traceback.format_exc())

    def save_spectrum_(self):
        try:
            file, _ = QFileDialog.getSaveFileName(self, 'Save File', f"{self.spec.metadata['name']}.txt")
            if file!= "":
                self.addText(f'save {file}')
                self.spec.to_csv(file)
        except Exception:
            self.addText(traceback.format_exc())

    def print_spectrum_(self):
        try:
            self.addText(str(self.spec.table))
        except Exception:
            self.addText(traceback.format_exc())

    def plot_spectrum_(self):
        
        try:
            fig, ax = plt.subplots(figsize=(4,4), dpi=75)
            draw.spectrum(self.spec, ax=ax)
            fig.tight_layout()
            plt.show(block=False)
            self.addText('plot_spectrum')
        
        except Exception:
            self.addText(traceback.format_exc())
    
    def plot_van_krevelen(self):

        try:
            fig, ax = plt.subplots(figsize=(4,4), dpi=75)
            draw.vk(self.spec, ax=ax)
            fig.tight_layout()
            plt.show(block=False)
            self.addText('plot_vk')
        
        except Exception:
            self.addText(traceback.format_exc())

    def assign_(self):
        
        try:
            ppm = float(self.assign_error_ppm.text())

            mod = self.mode.currentText()

            if mod == 'negative':
                sign = '-'
            else:
                sign = '+'
            
            if self.rules.isChecked():
                rul = True
            else:
                rul = False

            if self.elems != self.gdf_elems:
                self.gdf_elems = copy.deepcopy(self.elems)
                self.gdf = brutto_gen(elems = self.elems, rules=rul)

            self.spec = self.spec.assign(generated_bruttos_table=self.gdf, rel_error=ppm, sign=sign)
            self.addText('assigned')
        except Exception:
            self.addText(traceback.format_exc())

    def show_assign_error_(self):
        
        try:
            mod = self.mode.currentText()

            if mod == 'negative':
                sign = '-'
            else:
                sign = '+'
            self.spec = self.spec.calc_mass()
            self.spec = self.spec.calc_error(sign=sign)
            draw.show_error(self.spec)
            plt.show(block=False)
            self.addText('show assign error')
        
        except Exception:
            self.addText(traceback.format_exc())

    def remove_C_13_(self):
        
        try:
            self.spec = self.spec.filter_by_C13(remove=True)
            self.addText('peaks that dont have 13C isotope peak are removed')
        
        except Exception:
            self.addText(traceback.format_exc())

    def duplicates_(self):
        
        try:
            self.spec = self.spec.merge_duplicates()
            self.addText('duplicates are merged')
        
        except Exception:
            self.addText(traceback.format_exc()) 

    def list_2_clicked_(self):

        try:
            value = str(self.listWidget_2.currentItem().text())
            self.temp_element = value.split(" ")[0]

        except Exception:
            self.addText(traceback.format_exc())

    def refresh_list_2(self):
        try:
            self.listWidget_2.clear()

            for i, el in enumerate(self.elems):
                self.listWidget_2.insertItem(i, f'{el} [{self.elems[el][0]},{self.elems[el][1]-1}]')

        except Exception:
            self.addText(traceback.format_exc())

    def add_element_(self):

        try:
            ele = self.assign_element.text()
            isotop = self.assign_isotope.text()
            if isotop != '':
                ele = ele + '_' + isotop

            r_min = int(self.assign_range_min.text())
            r_max = int(self.assign_range_max.text()) + 1

            self.elems[ele]=(r_min, r_max)
        
            self.refresh_list_2()
        
        except Exception:
            self.addText(traceback.format_exc())
    
    def remove_element_(self):

        try:
            value = str(self.listWidget_2.currentItem().text())
            self.temp_element = value.split(" ")[0]
            del self.elems[self.temp_element]
            self.refresh_list_2()
        
        except Exception:
            self.addText(traceback.format_exc())

    def reset_element_(self):
        
        try:
            self.elems = {'C':(4, 51),'H':(4, 101),'O':(0,26), 'N':(0,4), 'S':(0,3)}
            self.refresh_list_2()
        
        except Exception:
            self.addText(traceback.format_exc())

    def generate_gdf_(self):
        
        try:
            if self.rules.isChecked():
                rul = True
            else:
                rul = False
                
            self.addText('gen gdf')
            self.gdf = brutto_gen(elems = self.elems, rules = rul)
            self.addText(str(self.gdf))

        except Exception:
            self.addText(traceback.format_exc())

    def gen_tmds_(self):

        try:
            p = self.tmds_p.text()
            n = self.max_tmds.text()

            if self.tmds_c13.isChecked():
                c13 = True
            else:
                c13 = False

            if n == '':
                n = None
            else:
                n = int(n)

            if p == '':
                p = 0.2
            else:
                p = float(p)
                
            self.tmds = Tmds(self.spec).calc(p=p, C13_filter=c13)

            if self.rules.isChecked():
                rul = True
            else:
                rul = False

            if self.elems != self.gdf_elems:
                self.gdf_elems = copy.deepcopy(self.elems)
                self.gdf = brutto_gen(elems = self.elems, rules=rul)

            self.tmds = self.tmds.assign(self.gdf, max_num=n)
            self.tmds = self.tmds.calc_mass()

            draw.spectrum(self.tmds)
            plt.show(block=False)

            self.addText(str(self.tmds.table))
        
        except Exception:
            self.addText(traceback.format_exc())

    def assign_by_tmds_(self):

        try:
            p = self.tmds_p.text()
            n = self.max_tmds.text()

            if self.tmds_c13.isChecked():
                c13 = True
            else:
                c13 = False

            if n == '':
                n = None
            else:
                n = int(n)

            if p == '':
                p = 0.2
            else:
                p = float(p)

            self.addText('it is takes a lot of time')
            self.spec = assign_by_tmds(spec=self.spec, tmds_spec=self.tmds, p=p, max_num=n, C13_filter=c13)
            self.addText('done')
        
        except Exception:
            self.addText(traceback.format_exc())

    def recallibrate_(self):
        
        try:
            mod = self.mode.currentText()

            if mod == 'negative':
                sign = '-'
            else:
                sign = '+'
            self.spec = recallibrate(self.spec, self.err, mode=sign)
            self.addText('recallibrate')
        
        except Exception:
            self.addText(traceback.format_exc())

    def load_etalon_(self):
        
        try:
            file, _ = QFileDialog.getOpenFileName(self, 'Open File')
            if file!="":
                self.addText(file)
                self.etalon = Spectrum.read_csv(filename=file,
                                    mapper={'mass':'mass', 'intensity':'intensity'},
                                    take_only_mz=False,
                                    sep=','
                                    )
                self.addText('load etalon')
            
        except Exception:
            self.addText(traceback.format_exc())

    def calc_recal_by_etalon_(self):
        
        try:
            self.err = ErrorTable().etalon_error(spec=self.spec, etalon=self.etalon)
            self.addText('calc_recal_by_etalon')
            plt.show(block=False)
        
        except Exception:
            self.addText(traceback.format_exc())

    def calc_self_recal_by_assign_(self):

        try:
            mod = self.mode.currentText()

            if mod == 'negative':
                sign = '-'
            else:
                sign = '+'
            self.err = ErrorTable().assign_error(self.spec, mode=sign)
            self.addText('calc_self_recal_by_assign')
            plt.show(block=False)
        
        except Exception:
            self.addText(traceback.format_exc())

    def calc_self_recal_by_mdiff_(self):

        try:
            self.err = ErrorTable().massdiff_error(self.spec)
            self.addText('calc_self_recal_by_mdiff')
            plt.show(block=False)
        
        except Exception:
            self.addText(traceback.format_exc())

    def load_error_(self):
        
        try:
            file, _ = QFileDialog.getOpenFileName(self, 'Open File')
            if file != "":
                self.addText(file)
                error = pd.read_csv(file)
                self.err = ErrorTable(error)
                self.addText('load_error')
        
        except Exception:
            self.addText(traceback.format_exc())

    def save_error_(self):

        try:
            file, _ = QFileDialog.getSaveFileName(self, 'Save File', "Text file *.txt")
            self.addText(f'save {file}')
            self.err.table.to_csv(file, index=False)
        
        except Exception:
            self.addText(traceback.format_exc())        
    
    def show_error_(self):
        
        try:
            self.err.show_error()
            self.addText('show_error')
            plt.show(block=False)
        
        except Exception:
            self.addText(traceback.format_exc()) 

    def extrapolate_(self):
        
        try:
            r_min = int(self.spec['mass'].min())
            r_max = int(self.spec['mass'].max())
            self.err = self.err.extrapolate(ranges=(r_min,r_max))
            self.addText('err_extra')
        
        except Exception:
            self.addText(traceback.format_exc()) 

    def range_(self):

        try:
            r_min = int(self.recal_range_min.text())
            r_max = int(self.recal_range_max.text())
            self.err.table = self.err.table.loc[(self.err.table['mass'] > r_min) & ((self.err.table['mass'] < r_max))]
            self.err = self.err.extrapolate(ranges=(r_min,r_max))
            self.addText('range error')
        
        except Exception:
            self.addText(traceback.format_exc()) 

    def load_background_(self):
        try:
            file, _ = QFileDialog.getOpenFileName(self, 'Open File')
            if file:
                self.addText(file)

            self.back = Spectrum.read_csv(filename=file,
                                mapper={'mass':'mass', 'intensity':'intensity'},
                                take_only_mz=False,
                                sep=','
                                )
        
        except Exception:
            self.addText(traceback.format_exc())

    def remove_background_(self):
        try:
            temp = self.spec.metadata
            self.spec = self.spec - self.back
            self.spec.metadata = temp
            self.spec.metadata.add({f'background_removed':{self.back.metadata['name']}})
            self.addText('background is removed')
        
        except Exception:
            self.addText(traceback.format_exc())

    def add_bufer_(self):

        try:
            self.specs_list.append(copy.deepcopy(self.spec))
            self.listWidget.insertItem(self.listWidget.count(), self.specs_list[-1].metadata['name'])
            self.listWidget.item(self.listWidget.count()-1).setCheckState(0)
            self.listWidget.setCurrentRow(self.listWidget.count()-1)
            self.specs_list[-1].metadata['color'] = def_colors[len(self.specs_list)%10]
            self.specs_list[-1].metadata['alpha'] = 0.2
            
        except Exception:
            self.addText(traceback.format_exc())

    def multi_load_(self):
        try:
            files, _ = QFileDialog.getOpenFileNames(self, 'Open File') 
            if files:
                init_len = len(self.specs_list)
                for i, file in enumerate(files):
                    self.specs_list.append(Spectrum.read_csv(file))
                    self.listWidget.insertItem(i+init_len, self.specs_list[-1].metadata['name'])
                    self.listWidget.item(i+init_len).setCheckState(0)
                    self.specs_list[-1].metadata['color'] = def_colors[len(self.specs_list)%10]
                    self.specs_list[-1].metadata['alpha'] = 0.2

            self.addText(f'{len(files)} spectra are loaded')

        except Exception:
            self.addText(traceback.format_exc())

    def save_(self):

        try:
            file = str(QFileDialog.getExistingDirectory(self, "Select Directory"))

            if 'out' not in os.listdir(file):
                path = os.path.join(file, "out")
                os.mkdir(path)

            text =''
            for i, spec in enumerate(self.specs_list):
                if self.listWidget.item(i).checkState() == 2:
                    text += f'{self.specs_list[i].metadata["name"]}\n'
                    spec.to_csv(f'{file}/out/{spec.metadata["name"]}.txt')

            self.addText(f'Spectrum are saved in {file}/out\n{text}')
        
        except Exception:
            self.addText(traceback.format_exc())
            
    def remove_(self):

        try:
            text =''

            i = 0
            while i!= self.listWidget.count():
                if self.listWidget.item(i).checkState() == 2:
                    text += f'{self.specs_list[i].metadata["name"]}\n'
                    del self.specs_list[i]
                    self.listWidget.takeItem(i)
                else:
                    i += 1
            
            self.addText(f'Spectrum are removed\n{text}')
        
        except Exception:
            self.addText(traceback.format_exc())

    def add_operate(self):
        self.operate(op='add')
    
    def sub_operate(self):
        self.operate(op='sub')

    def and_operate(self):
        self.operate(op='and')

    def xor_operate(self):
        self.operate(op='xor')

    def int_sub_operate(self):
        self.operate(op='int_sub')

    def operate(self, op):

        try:
            self.form_temp_list()
            if len(self.temp_list) < 2:
                raise Exception('Operation requires at least two spectra')

            obj = copy.deepcopy(self.temp_list)
            for i, spec in enumerate(obj):
                obj[i] = spec.calc_mass()
                obj[i].table = obj[i].table.dropna()

            temp_spec = obj[0]
            if op == 'and':
                for i in range(1, len(obj)):
                    temp_spec = temp_spec & obj[i]
                self.addText('and')
            elif op == 'add':
                for i in range(1, len(obj)):
                    temp_spec = temp_spec + obj[i]
                self.addText('add')
            elif op == 'sub':
                for i in range(1, len(obj)):
                    temp_spec = temp_spec - obj[i]
                self.addText('sub')
            elif op == 'xor':
                for i in range(1, len(obj)):
                    temp_spec = temp_spec ^ obj[i]
                self.addText('xor')
            elif op == 'int_sub':
                for i in range(1, len(obj)):
                    temp_spec = temp_spec.intens_sub(obj[i])
                self.addText('int_sub')

            self.specs_list.append(copy.deepcopy(temp_spec))
            self.listWidget.insertItem(self.listWidget.count(), self.specs_list[-1].metadata['name'])
            self.listWidget.item(self.listWidget.count()-1).setCheckState(0)
            self.listWidget.setCurrentRow(self.listWidget.count()-1)
            self.specs_list[-1].metadata['color'] = def_colors[len(self.specs_list)%10]
            self.specs_list[-1].metadata['alpha'] = 0.2

        except Exception:
            self.addText(traceback.format_exc())

    def venn_operate(self):
        
        try:
            self.form_temp_list()

            obj = copy.deepcopy(self.temp_list)
            sets = []
            for i, spec in enumerate(obj):
                obj[i] = spec.calc_mass()
                obj[i].table = obj[i].table.dropna()
                sets.append(obj[i].table['calc_mass'].dropna().values)

            fig, ax = self.get_fig_ax()
            labels = (self.temp_list[i].metadata['name'] for i in range(len(self.temp_list)))                
            
            if len(self.temp_list) == 2:                    
                venn2([set(sets[0]), set(sets[1])], set_labels=labels, ax=ax)

            elif len(self.temp_list) == 3:
                venn3([set(sets[0]), set(sets[1]), set(sets[2])], set_labels=labels, ax=ax)

            else:
                raise Exception(f"This can be done for two or three spectra, not for {len(self.temp_list)}")
            
            self.addText('venn')

            fig.tight_layout()
            plt.show(block=False)        
            if self.path_img is not None and self.save_box.isChecked():
                form = self.format.currentText()
                name = self.temp_name_img
                plt.savefig(f'{self.path_img}/{name}.{form}')
                self.temp_name_img += 1

        except Exception:
            self.addText(traceback.format_exc())

    def check_all_(self):
        try:
            for i in range(self.listWidget.count()):
                self.listWidget.item(i).setCheckState(2)
        except Exception:
            self.addText(traceback.format_exc())

    def uncheck_all_(self):
        try:
            for i in range(self.listWidget.count()):
                self.listWidget.item(i).setCheckState(0)
        except Exception:
            self.addText(traceback.format_exc())

    def up_(self):
        try:
            value = self.listWidget.currentRow()
            if value > 0:
                temp_spec = self.specs_list[value]
                del self.specs_list[value]
                self.specs_list.insert(value-1, temp_spec)

                currentItem = self.listWidget.takeItem(value)
                self.listWidget.insertItem(value - 1, currentItem)

                self.listWidget.setCurrentRow(value-1)

        except Exception:
            self.addText(traceback.format_exc())

    def down_(self):
        try:
            value = self.listWidget.currentRow()
            if value < len(self.specs_list)-1:
                temp_spec = self.specs_list[value]
                del self.specs_list[value]
                self.specs_list.insert(value+1, temp_spec)

                currentItem = self.listWidget.takeItem(value)
                self.listWidget.insertItem(value + 1, currentItem)
                
                self.listWidget.setCurrentRow(value+1)

        except Exception:
            self.addText(traceback.format_exc())

    def form_temp_list(self):

        try:
            self.temp_list = SpectrumList()
            self.plot_alpha = []
            self.plot_color = []
            for i, spec in enumerate(self.specs_list):
                if self.listWidget.item(i).checkState() == 2:
                    self.temp_list.append(spec)
                    self.plot_alpha.append(spec.metadata["alpha"])
                    self.plot_color.append(spec.metadata["color"])
        
        except Exception:
            self.addText(traceback.format_exc())

    def list_double_clicked_(self):

        try:
            value = self.listWidget.currentRow()
            dlg = ListDialog(spec = self.specs_list[value])
            result = dlg.exec_()
            if result:
                self.specs_list[value] = dlg.spec
                self.listWidget.item(value).setText(self.specs_list[value].metadata['name'])

        except Exception:
            self.addText(traceback.format_exc())

    def calculate_(self):

        try:
            for i, spec in enumerate(self.specs_list):
                if self.listWidget.item(i).checkState() == 2:
                    self.specs_list[i] = self.specs_list[i].calc_all_metrics().drop_unassigned()
            
            self.addText('Spectrum are calculated')
        
        except Exception:
            self.addText(traceback.format_exc())

    def normalize_(self):

        try:
            how = self.nomalize_box.currentText()

            for i, spec in enumerate(self.specs_list):
                if self.listWidget.item(i).checkState() == 2:
                    self.specs_list[i] = self.specs_list[i].normalize(how=how)
            
            self.addText('Spectrum are normalized')
        
        except Exception:
            self.addText(traceback.format_exc())

    def count_(self):

        try:
            self.form_temp_list()
            obj = copy.deepcopy(self.temp_list)
            
            func = self.func.currentText()
            value = self.cut_line.text()
            sign = self.cut_box_2.currentText()
            how = self.cut_box.currentText()

            if value != "":
                value = float(value)
                if sign == '<':
                    for i, spec in enumerate(obj):
                        obj[i].table = spec.table.loc[spec.table[how] > value]

                elif sign == '>':
                    for i, spec in enumerate(obj):
                        obj[i].table = spec.table.loc[spec.table[how] < value]
                
                elif sign == '=':
                    for i, spec in enumerate(obj):
                        obj[i].table = spec.table.loc[spec.table[how] == value]

            self.temp_df = obj.get_mol_metrics(func=func)
            self.addText(str(self.temp_df))
        
        except Exception:
            self.addText(traceback.format_exc())

    def cut_(self):

        try:
            self.form_temp_list()
            obj = self.temp_list
            
            how = self.cut_box.currentText()
            value = self.cut_line.text()
            sign = self.cut_box_2.currentText()

            if value == "":
                raise Exception('fill value')
            else:
                value = float(value)
            
            if sign == '<':
                for i, spec in enumerate(obj):
                    obj[i].table = spec.table.loc[spec.table[how] > value]

            elif sign == '>':
                for i, spec in enumerate(obj):
                    obj[i].table = spec.table.loc[spec.table[how] < value]
            
            elif sign == '=':
                for i, spec in enumerate(obj):
                    obj[i].table = spec.table.loc[spec.table[how] == value]

            self.addText(f'specs are cut by {how} with {sign}')
        
        except Exception:
            self.addText(traceback.format_exc())

    def classes_(self):

        try:
            self.form_temp_list()
            obj = self.temp_list

            fig, ax = self.get_fig_ax()

            cl = self.classes_choise.currentText()

            df = obj.get_mol_density(how=cl)
            obj.draw_mol_density(mol_density=df, ax=ax)

            fig.tight_layout()
            plt.show(block=False)
            
            if self.path_img is not None and self.save_box.isChecked():
                form = self.format.currentText()
                name = self.temp_name_img
                plt.savefig(f'{self.path_img}/{name}.{form}')
                self.temp_name_img += 1

            self.temp_df = df
            self.addText(str(df))
        
        except Exception:
            self.addText(traceback.format_exc())

    def squares_(self):
        
        try:
            self.form_temp_list()
            obj = self.temp_list

            spec_num = len(obj)
            if spec_num < 1:
                raise Exception("no spectrum for plot. Try to add by check")
            if spec_num == 1:
                fig, ax = self.get_fig_ax()
                self.squares = obj[0].get_squares_vk(ax=ax, draw=True)
                fig.tight_layout()
                plt.show(block=False)
            
            self.temp_df = obj.get_square_vk()
            self.addText(str(self.temp_df.head(20)))
        
        except Exception:
            self.addText(traceback.format_exc())

    def dbe_vs_no(self):

        try:
            self.form_temp_list()
            obj = self.temp_list    

            fig, ax = self.get_fig_ax()

            omin = self.no_min.text()
            omax = self.no_max.text()
            lim = None
            if omin!="" and omax!="":
                lim = (int(omin), int(omax))
            elif omin!="" or omax!="":
                raise Exception('it is neceserry to fill both value for nO range, or fill it empty for default diapasone')

            spec_num = len(obj)
            if spec_num < 1:
                raise Exception("no spectrum for plot. Try to add by click")

            col = self.plot_color
            
            dt = []
            for i, spec in enumerate(obj):
                val = spec.get_dbe_vs_o(ax=ax, olim=lim, c=col[i])                   
                dt.append([spec.metadata['name'], col[i], np.round(val[0], 2)])
            self.temp_df = pd.DataFrame(data = dt, columns=['name', 'color','a'])            
            self.addText(str(self.temp_df))

            ax = self.restrict_ax(ax)
            fig.tight_layout()
            plt.show(block=False)

            if self.path_img is not None and self.save_box.isChecked():
                form = self.format.currentText()
                name = self.temp_name_img
                plt.savefig(f'{self.path_img}/{name}.{form}')
                self.temp_name_img += 1
        
        except Exception:
            self.addText(traceback.format_exc())

    def simmilarity_(self):

        try:
            self.form_temp_list()
            obj = self.temp_list

            sim = self.similarity_metric.currentText()
            
            matrix = obj.get_simmilarity(mode=sim)
            names = [i.metadata['name'] for i in obj]
            self.temp_df = pd.DataFrame(data = matrix, columns= names, index=names)
            self.addText(str(self.temp_df))

            fig, ax = self.get_fig_ax()
            obj.draw_simmilarity(values=matrix ,mode=sim, ax=ax)
            fig.tight_layout()
            plt.show(block=False)

            if self.path_img is not None and self.save_box.isChecked():
                form = self.format.currentText()
                name = self.temp_name_img
                plt.savefig(f'{self.path_img}/{name}.{form}')
                self.temp_name_img += 1
        
        except Exception:
            self.addText(traceback.format_exc())

    def save_csv_(self):

        try:
            file, _ = QFileDialog.getSaveFileName(self, 'Save File', f"data.csv")
            if file != '':
                self.addText(f'save {file}')
                self.temp_df.to_csv(file)
        
        except Exception:
            self.addText(traceback.format_exc())

    def scatter_(self):

        try:
            fig, ax = self.get_fig_ax()

            self.form_temp_list()
            spec_num = len(self.temp_list)

            col = self.plot_color

            if len(self.temp_list)==1:
                col = [None]

            alp = self.plot_alpha

            xs = self.scatter_box_1.currentText()
            ys = self.sactter_box_2.currentText()
            volume = self.sactter_box_3.currentText()
            
            s = self.size_volume.text()
            if s == '':
                s = None
            else:
                s = float(s)

            pow = self.pow.text()
            if pow == '':
                pow = None
            else:
                pow = float(pow)

            out = ''

            for i, spec in enumerate(self.temp_list):           
                draw.scatter(spec, x=xs, y=ys, volume = volume, ax=ax, alpha=alp[i], color=col[i], size=s, size_power=pow)
                out = out + spec.metadata['name'] + ' color ' + str(col[i]) + '\n'
            
            if len(self.temp_list)>1:
                ax.set_title(" ")

            hc_oc = False
            if xs=='O/C' and ys=='H/C':
                hc_oc = True

            ax = self.restrict_ax(ax, hc_oc=hc_oc)
            fig.tight_layout()
            plt.show(block=False)
            
            if self.path_img is not None and self.save_box.isChecked():
                form = self.format.currentText()
                name = self.temp_name_img
                plt.savefig(f'{self.path_img}/{name}.{form}')
                self.temp_name_img += 1

            self.addText(out)
        
        except Exception:
            self.addText(traceback.format_exc())

    def spectrum_(self):

        try:
            fig, ax = self.get_fig_ax()

            self.form_temp_list()
            spec_num = len(self.temp_list)

            col = self.plot_color

            out = ''
            for i, spec in enumerate(self.temp_list):                   
                draw.spectrum(spec, ax=ax, color=col[i])
                out = out + spec.metadata['name'] + ' color ' + str(col[i]) + '\n'

            if len(self.temp_list) > 1:
                ax.set_title(" ")

            ax = self.restrict_ax(ax)
            fig.tight_layout()
            plt.show(block=False)
            
            if self.path_img is not None and self.save_box.isChecked():
                form = self.format.currentText()
                name = self.temp_name_img
                plt.savefig(f'{self.path_img}/{name}.{form}')
                self.temp_name_img += 1

            self.addText(out)
        
        except Exception:
            self.addText(traceback.format_exc())

    def scatter_dens(self):
            
        try:
            dpi = int(self.dpi_line.text())
            size1 = int(self.size_line.text())
            size2 = int(self.size_line_2.text())

            fig = plt.figure(figsize=(size1,size2), dpi=dpi)
            gs = GridSpec(4, 4)

            ax = fig.add_subplot(gs[1:4, 0:3])
            ax_x = fig.add_subplot(gs[0,0:3])
            ax_y = fig.add_subplot(gs[1:4, 3])

            self.form_temp_list()
            spec_num = len(self.temp_list)
            if spec_num < 1:
                raise Exception("no spectrum for plot. Try to add by click")

            col = self.plot_color

            if len(self.temp_list)==1:
                col = [None]

            alp = self.plot_alpha

            xs = self.scatter_box_1.currentText()
            ys = self.sactter_box_2.currentText()
            volume = self.sactter_box_3.currentText()
            s = self.size_volume.text()
            if s == '':
                s = None
            else:
                s = float(s)

            pow = self.pow.text()
            if pow == '':
                pow = None
            else:
                pow = float(pow)

            out = ''
            for i, spec in enumerate(self.temp_list):
                draw.scatter_density(spec, x=xs, y=ys, ax=[ax, ax_x, ax_y], volume = volume, color=col[i], alpha=alp[i], size=s, size_power=pow, title=False)              
                out = out + spec.metadata['name'] + ' color ' + str(col[i]) + '\n'
            self.addText(out)

            hc_oc = False
            if xs=='O/C' and ys=='H/C':
                hc_oc = True

            x_min = self.x1.text()
            if x_min == '':
                x_min = None
            else:
                x_min = float(x_min)

            x_max = self.x2.text()
            if x_max == '':
                x_max = None
            else:
                x_max = float(x_max)

            y_min = self.y1.text()
            if y_min == '':
                y_min = None
            else:
                y_min = float(y_min)

            y_max = self.y2.text()
            if y_max == '':
                y_max = None
            else:
                y_max = float(y_max)

            if x_min is not None and x_max is not None:
                ax.set_xlim(x_min, x_max)
                ax_x.set_xlim(x_min, x_max)
            elif x_min is None and x_max is None and hc_oc:
                ax.set_xlim(0, 1)
                ax_x.set_xlim(0, 1)
            if y_min is not None and y_max is not None:
                ax.set_ylim(y_min, y_max)
                ax_y.set_ylim(y_min, y_max)
            elif y_min is None and y_max is None and hc_oc:
                ax.set_ylim(0, 2.2)
                ax_y.set_ylim(0, 2.2)
            
            fig.tight_layout()
            plt.show(block=False)

            if self.path_img is not None and self.save_box.isChecked():
                form = self.format.currentText()
                name = self.temp_name_img
                plt.savefig(f'{self.path_img}/{name}.{form}')
                self.temp_name_img += 1
        
        except Exception:
            self.addText(traceback.format_exc())

    def density_(self):

        try:
            fig, ax = self.get_fig_ax()
            ax = self.restrict_ax(ax, hc_oc=False)

            self.form_temp_list()
            spec_num = len(self.temp_list)

            col = self.plot_color

            xs = self.density_box.currentText()
        
            out = ''
            for i, spec in enumerate(self.temp_list):                   
                draw.density(spec, col=xs, ax=ax, color=col[i])
                out = out + spec.metadata['name'] + ' color ' + str(col[i]) + '\n'

            if len(self.temp_list)>1:
                ax.set_title(" ")

            ax.set_ylim(auto=True)

            fig.tight_layout()
            plt.show(block=False)
            
            if self.path_img is not None and self.save_box.isChecked():
                form = self.format.currentText()
                name = self.temp_name_img
                plt.savefig(f'{self.path_img}/{name}.{form}')
                self.temp_name_img += 1

            self.addText(out)
        
        except Exception:
            self.addText(traceback.format_exc())

    def path_(self):

        try:
            file = str(QFileDialog.getExistingDirectory(self, "Select Directory"))
            
            self.path_img = os.path.join(file, "out_img")    
            if 'out_img' not in os.listdir(file):
                os.mkdir(self.path_img)

            self.addText(f'add path to save fugures: {self.path_img}')
        
        except Exception:
            self.addText(traceback.format_exc())

    def get_fig_ax(self):
        dpi = int(self.dpi_line.text()) 
        size1 = int(self.size_line.text())
        size2 = int(self.size_line_2.text())
        fig, ax = plt.subplots(figsize=(size1, size2), dpi=dpi)
        return fig, ax

    def restrict_ax(self, ax, hc_oc=False):

        x_min = self.x1.text()
        if x_min == '':
            x_min = None
        else:
            x_min = float(x_min)

        x_max = self.x2.text()
        if x_max == '':
            x_max = None
        else:
            x_max = float(x_max)

        y_min = self.y1.text()
        if y_min == '':
            y_min = None
        else:
            y_min = float(y_min)

        y_max = self.y2.text()
        if y_max == '':
            y_max = None
        else:
            y_max = float(y_max)

        if x_min is not None and x_max is not None:
            ax.set_xlim(x_min, x_max)
        elif x_min is None and x_max is None and hc_oc:
            ax.set_xlim(0, 1)
        if y_min is not None and y_max is not None:
            ax.set_ylim(y_min, y_max)
        elif y_min is None and y_max is None and hc_oc:
            ax.set_ylim(0, 2.2)
        return ax


if __name__ == '__main__':

    app = QtWidgets.QApplication(sys.argv)
    window = App()
    window.show()
    app.exec_()